# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# This is referred from Redfish standard schema.
# http://redfish.dmtf.org/schemas/v1/Fabric.v1_0_4.json

from sushy.resources import base
from sushy.resources import common
from sushy.resources.fabric import endpoint as fab_endpoint
from sushy.resources import mappings as res_maps
from sushy import utils

import logging

LOG = logging.getLogger(__name__)


class Fabric(base.ResourceBase):
    """Fabric resource

    The Fabric represents a simple fabric consisting of one or more
    switches, zero or more endpoints, and zero or more zones.
    """

    identity = base.Field('Id', required=True)
    """Identifier for the fabric"""

    name = base.Field('Name', required=True)
    """The fabric name"""

    description = base.Field('Description')
    """The fabric description"""

    max_zones = base.Field('MaxZones', adapter=utils.int_or_none)
    """The maximum number of zones the switch can currently configure"""

    status = common.StatusField('Status')
    """The fabric status"""

    fabric_type = base.MappedField('FabricType',
                                   res_maps.PROTOCOL_TYPE_VALUE_MAP)
    """The protocol being sent over this fabric"""

    def __init__(self, connector, identity, redfish_version=None,
                 registries=None):
        """A class representing a Fabric

        :param connector: A Connector instance
        :param identity: The identity of the Fabric resource
        :param redfish_version: The version of RedFish. Used to construct
            the object according to schema of the given version.
        :param registries: Dict of Redfish Message Registry objects to be
            used in any resource that needs registries to parse messages
        """
        super(Fabric, self).__init__(
            connector, identity, redfish_version, registries)

    @property
    @utils.cache_it
    def endpoints(self):
        return fab_endpoint.EndpointCollection(
            self._conn, utils.get_sub_resource_path_by(self, 'Endpoints'),
            self.redfish_version, self.registries)


class FabricCollection(base.ResourceCollectionBase):

    @property
    def _resource_type(self):
        return Fabric

    def __init__(self, connector, path, redfish_version=None, registries=None):
        """A class representing a FabricCollection

        :param connector: A Connector instance
        :param path: The canonical path to the Fabric collection resource
        :param redfish_version: The version of RedFish. Used to construct
            the object according to schema of the given version.
        :param registries: Dict of Redfish Message Registry objects to be
            used in any resource that needs registries to parse messages
        """
        super(FabricCollection, self).__init__(
            connector, path, redfish_version, registries)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

# This is described in Redfish specification section "Asynchronous operations"
# www.dmtf.org/sites/default/files/standards/documents/DSP0266_1.7.0.pdf


from datetime import datetime
from datetime import timedelta
import logging

from dateutil import parser

from sushy.resources import base

LOG = logging.getLogger(__name__)


class TaskMonitor(base.ResourceBase):

    def __init__(self,
                 connector,
                 path='',
                 redfish_version=None):
        """A class representing a Redfish Task Monitor

        :param connector: A Connector instance
        :param path: sub-URI path to the resource.
        :param redfish_version: The version of Redfish. Used to construct
            the object according to schema of the given version.
        """
        super(TaskMonitor, self).__init__(connector, path, redfish_version)
        self._retry_after = None
        self._location_header = None
        self._in_progress = True
        self._response = None

    @staticmethod
    def _to_datetime(retry_after):
        if isinstance(retry_after, int) or retry_after.isdigit():
            # Retry-After: 120
            return datetime.now() + timedelta(seconds=int(retry_after))
        else:
            # Retry-After: Fri, 31 Dec 1999 23:59:59 GMT
            return parser.parse(retry_after)

    def set_retry_after(self, value):
        """Set the time the client should wait before querying the task status

        :param value: The value of the Retry-After header, which can be the
                      number of seconds to wait or an `HTTP-date` string as
                      defined by RFC 7231
        :return: The TaskMonitor object
        """
        self._retry_after = self._to_datetime(value or 1)
        return self

    @property
    def retry_after(self):
        """Time the client should wait before querying the task status

        :return: The Retry-After time in `datetime` format
        """
        return self._retry_after

    @property
    def sleep_for(self):
        """Seconds the client should wait before querying the operation status

        :return: The number of seconds to wait
        """
        return max(0, (self._retry_after - datetime.now()).total_seconds())

    @property
    def location_header(self):
        """The Location header returned from the GET on the Task Monitor

        :return: The Location header (an absolute URL)
        """
        return self._location_header

    @property
    def in_progress(self):
        """Checks the status of the async task

        :return: True if the async task is still in progress, False otherwise
        """
        if not self._in_progress:
            return False
        r = self._conn.get(self._path)
        self._response = r
        self._location_header = r.headers.get('location')
        if r.status_code == 202:
            self.set_retry_after(r.headers.get('retry-after'))
        else:
            self._in_progress = False
        return self._in_progress

    @property
    def response(self):
        """The response from the last TaskMonitor in_progress check

        :return: The `requests` response object or None
        """
        return self._response



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging

from sushy.resources import base


LOG = logging.getLogger(__name__)


class OEMResourceBase(base.ResourceBase):

    def __init__(self,
                 connector,
                 path='',
                 redfish_version=None,
                 registries=None,
                 reader=None):
        """Class representing an OEM vendor extension

        :param connector: A Connector instance
        :param path: sub-URI path to the resource.
        :param redfish_version: The version of Redfish. Used to construct
            the object according to schema of the given version.
        :param registries: Dict of Redfish Message Registry objects to be
            used in any resource that needs registries to parse messages
        """
        self._parent_resource = None
        self._vendor_id = None

        super(OEMResourceBase, self).__init__(
            connector, path, redfish_version, registries, reader)

    def set_parent_resource(self, parent_resource, vendor_id):
        self._parent_resource = parent_resource
        self._vendor_id = vendor_id
        # NOTE(etingof): this is required to pull OEM subtree
        self.invalidate(force_refresh=True)
        return self

    def _parse_attributes(self, json_doc):
        """Parse the attributes of a resource.

        Parsed JSON fields are set to `self` as declared in the class.

        :param json_doc: parsed JSON document in form of Python types
        """
        oem_json = json_doc.get(
            'Oem', {}).get(self._vendor_id, {})

        # NOTE(etingof): temporary copy Actions into Oem subtree for parsing
        # all fields at once

        oem_json = oem_json.copy()

        oem_actions_json = {
            'Actions': json_doc.get(
                'Actions', {}).get('Oem', {})
        }

        oem_json.update(oem_actions_json)

        super(OEMResourceBase, self)._parse_attributes(oem_json)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


# Power Supply Types
POWER_SUPPLY_TYPE_UNKNOWN = 'unknown'
"""The power supply type cannot be determined."""

POWER_SUPPLY_TYPE_AC = 'ac'
"""Alternating Current (AC) power supply."""

POWER_SUPPLY_TYPE_DC = 'dc'
"""Direct Current (DC) power supply."""

POWER_SUPPLY_TYPE_ACDC = 'acdc'
"""Power Supply supports both DC or AC."""

# Line Input Voltage Types
LINE_INPUT_VOLTAGE_TYPE_UNKNOWN = 'unknown'
"""The power supply line input voltage tpye cannot be determined."""

LINE_INPUT_VOLTAGE_TYPE_ACLOW = 'aclowline'
"""100-127V AC input."""

LINE_INPUT_VOLTAGE_TYPE_ACMID = 'acmidline'
"""200-240V AC input."""

LINE_INPUT_VOLTAGE_TYPE_ACHIGH = 'achighline'
"""277V AC input."""

LINE_INPUT_VOLTAGE_TYPE_DCNEG48 = 'dcneg48v'
"""-48V DC input."""

LINE_INPUT_VOLTAGE_TYPE_DC380 = 'dc380v'
"""High Voltage DC input (380V)."""

LINE_INPUT_VOLTAGE_TYPE_AC120 = 'ac120v'
"""AC 120V nominal input."""

LINE_INPUT_VOLTAGE_TYPE_AC240 = 'ac240v'
"""AC 240V nominal input."""

LINE_INPUT_VOLTAGE_TYPE_AC277 = 'ac277v'
"""AC 277V nominal input."""

LINE_INPUT_VOLTAGE_TYPE_ACDCWIDE = 'acdcwiderange'
"""Wide range AC or DC input."""

LINE_INPUT_VOLTAGE_TYPE_ACWIDE = 'acwiderange'
"""Wide range AC input."""

LINE_INPUT_VOLTAGE_TYPE_DC240 = 'dc240v'
"""DC 240V nominal input."""

# Input Types
INPUT_TYPE_AC = 'ac'
"""Alternating Current (AC) input range."""

INPUT_TYPE_DC = 'dc'
"""Direct Current (DC) input range."""



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import mock
import stevedore

from sushy import exceptions
from sushy.resources import base as res_base
from sushy.resources.oem import base as oem_base
from sushy.resources.oem import common as oem_common
from sushy.tests.unit import base


class ContosoResourceOEMExtension(oem_base.OEMResourceBase):
    pass


class FauxResourceOEMExtension(oem_base.OEMResourceBase):
    pass


class ResourceOEMCommonMethodsTestCase(base.TestCase):

    def setUp(self):
        super(ResourceOEMCommonMethodsTestCase, self).setUp()
        # We use ExtensionManager.make_test_instance() and instantiate the
        # test instance outside of the test cases in setUp. Inside of the
        # test cases we set this as the return value of the mocked
        # constructor. Also note that this instrumentation has been done
        # only for one specific resource namespace which gets passed in the
        # constructor of ExtensionManager. Moreover, this setUp also enables
        # us to verify that the constructor is called correctly while still
        # using a more realistic ExtensionManager.
        contoso_ep = mock.Mock()
        contoso_ep.module_name = __name__
        contoso_ep.attrs = ['ContosoResourceOEMExtension']
        self.contoso_extn = stevedore.extension.Extension(
            'contoso', contoso_ep, lambda: ContosoResourceOEMExtension, None)
        self.contoso_extn_dup = stevedore.extension.Extension(
            'contoso_dup', contoso_ep,
            lambda: ContosoResourceOEMExtension, None)

        faux_ep = mock.Mock()
        faux_ep.module_name = __name__
        faux_ep.attrs = ['FauxResourceOEMExtension']
        self.faux_extn = stevedore.extension.Extension(
            'faux', faux_ep, lambda: FauxResourceOEMExtension, None)
        self.faux_extn_dup = stevedore.extension.Extension(
            'faux_dup', faux_ep, lambda: FauxResourceOEMExtension, None)

        self.fake_ext_mgr = (
            stevedore.extension.ExtensionManager.make_test_instance(
                [self.contoso_extn, self.faux_extn]))
        self.fake_ext_mgr2 = (
            stevedore.extension.ExtensionManager.make_test_instance(
                [self.contoso_extn_dup, self.faux_extn_dup]))

    def tearDown(self):
        super(ResourceOEMCommonMethodsTestCase, self).tearDown()
        if oem_common._global_extn_mgrs_by_resource:
            oem_common._global_extn_mgrs_by_resource = {}

    @mock.patch.object(stevedore, 'ExtensionManager', autospec=True)
    def test__create_extension_manager(self, ExtensionManager_mock):
        system_resource_oem_ns = 'sushy.resources.system.oems'
        ExtensionManager_mock.return_value = self.fake_ext_mgr

        result = oem_common._create_extension_manager(system_resource_oem_ns)

        self.assertEqual(self.fake_ext_mgr, result)
        ExtensionManager_mock.assert_called_once_with(
            system_resource_oem_ns, propagate_map_exceptions=True,
            on_load_failure_callback=oem_common._raise)

    @mock.patch.object(stevedore, 'ExtensionManager', autospec=True)
    def test__create_extension_manager_no_extns(self, ExtensionManager_mock):
        system_resource_oem_ns = 'sushy.resources.system.oems'
        ExtensionManager_mock.return_value.names.return_value = []

        self.assertRaisesRegex(
            exceptions.ExtensionError, 'No extensions found',
            oem_common._create_extension_manager,
            system_resource_oem_ns)

    @mock.patch.object(stevedore, 'ExtensionManager', autospec=True)
    def test__get_extension_manager_of_resource(self, ExtensionManager_mock):
        ExtensionManager_mock.return_value = self.fake_ext_mgr

        result = oem_common._get_extension_manager_of_resource('system')
        self.assertEqual(self.fake_ext_mgr, result)
        ExtensionManager_mock.assert_called_once_with(
            namespace='sushy.resources.system.oems',
            propagate_map_exceptions=True,
            on_load_failure_callback=oem_common._raise)
        ExtensionManager_mock.reset_mock()

        result = oem_common._get_extension_manager_of_resource('manager')
        self.assertEqual(self.fake_ext_mgr, result)
        ExtensionManager_mock.assert_called_once_with(
            namespace='sushy.resources.manager.oems',
            propagate_map_exceptions=True,
            on_load_failure_callback=oem_common._raise)
        for name, extension in result.items():
            self.assertTrue(name in ('contoso', 'faux'))
            self.assertTrue(extension in (self.contoso_extn,
                                          self.faux_extn))

    def test__get_resource_vendor_extension_obj_lazy_plugin_invoke(self):
        resource_instance_mock = mock.Mock()
        extension_mock = mock.MagicMock()
        extension_mock.obj = None

        mock_oem_resource = extension_mock.plugin.return_value

        result = oem_common._get_resource_vendor_extension_obj(
            extension_mock, resource_instance_mock, 'fish-n-chips')

        mock_clone_resource = resource_instance_mock.clone_resource
        mock_clone_resource.assert_called_once_with(mock_oem_resource)
        mock_ext = mock_clone_resource.return_value
        mock_ext.set_parent_resource.assert_called_once_with(
            resource_instance_mock, 'fish-n-chips')
        mock_ext = mock_ext.set_parent_resource.return_value
        self.assertEqual(result, mock_ext)

        extension_mock.reset_mock()

        # extension_mock.obj is not None anymore
        oem_common._get_resource_vendor_extension_obj(
            extension_mock, resource_instance_mock, 'fish-n-chips')

        self.assertFalse(extension_mock.plugin.called)

    @mock.patch.object(stevedore, 'ExtensionManager', autospec=True)
    def test_get_resource_extension_by_vendor(self, ExtensionManager_mock):
        oem_resource_mock = mock.Mock()
        oem_resource_mock.set_parent_resource = lambda *x: oem_resource_mock
        resource_instance_mock = mock.Mock()
        resource_instance_mock.clone_resource = lambda *x: oem_resource_mock
        ExtensionManager_mock.side_effect = [self.fake_ext_mgr,
                                             self.fake_ext_mgr2]

        result = oem_common.get_resource_extension_by_vendor(
            'system', 'Faux', resource_instance_mock)
        self.assertEqual(result, oem_resource_mock)
        ExtensionManager_mock.assert_called_once_with(
            'sushy.resources.system.oems', propagate_map_exceptions=True,
            on_load_failure_callback=oem_common._raise)
        ExtensionManager_mock.reset_mock()

        oem_resource_mock.obj = None
        result = oem_common.get_resource_extension_by_vendor(
            'system', 'Contoso', resource_instance_mock)
        self.assertEqual(result, oem_resource_mock)
        self.assertFalse(ExtensionManager_mock.called)
        ExtensionManager_mock.reset_mock()

        result = oem_common.get_resource_extension_by_vendor(
            'manager', 'Faux_dup', resource_instance_mock)
        self.assertEqual(result, oem_resource_mock)
        ExtensionManager_mock.assert_called_once_with(
            'sushy.resources.manager.oems', propagate_map_exceptions=True,
            on_load_failure_callback=oem_common._raise)
        ExtensionManager_mock.reset_mock()

        result = oem_common.get_resource_extension_by_vendor(
            'manager', 'Contoso_dup', resource_instance_mock)
        self.assertEqual(result, oem_resource_mock)
        self.assertFalse(ExtensionManager_mock.called)
        ExtensionManager_mock.reset_mock()

    @mock.patch.object(stevedore, 'ExtensionManager', autospec=True)
    def test_get_resource_extension_by_vendor_fail(
            self, ExtensionManager_mock):
        resource_instance_mock = mock.Mock(spec=res_base.ResourceBase)
        # ``fake_ext_mgr2`` has extension names as ``faux_dup``
        # and ``contoso_dup``.
        ExtensionManager_mock.return_value = self.fake_ext_mgr2

        self.assertRaisesRegex(
            exceptions.OEMExtensionNotFoundError,
            'No sushy.resources.system.oems OEM extension found '
            'by name "faux"',
            oem_common.get_resource_extension_by_vendor,
            'sushy.resources.system.oems', 'Faux', resource_instance_mock)



# Copyright 2017 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from sushy.resources.system import constants as sys_cons
from sushy import utils


RESET_SYSTEM_VALUE_MAP = {
    'On': sys_cons.RESET_ON,
    'ForceOff': sys_cons.RESET_FORCE_OFF,
    'GracefulShutdown': sys_cons.RESET_GRACEFUL_SHUTDOWN,
    'GracefulRestart': sys_cons.RESET_GRACEFUL_RESTART,
    'ForceRestart': sys_cons.RESET_FORCE_RESTART,
    'Nmi': sys_cons.RESET_NMI,
    'ForceOn': sys_cons.RESET_FORCE_ON,
    'PushPowerButton': sys_cons.RESET_PUSH_POWER_BUTTON,
}

RESET_SYSTEM_VALUE_MAP_REV = utils.revert_dictionary(RESET_SYSTEM_VALUE_MAP)

BOOT_SOURCE_TARGET_MAP = {
    'None': sys_cons.BOOT_SOURCE_TARGET_NONE,
    'Pxe': sys_cons.BOOT_SOURCE_TARGET_PXE,
    'Floppy': sys_cons.BOOT_SOURCE_TARGET_FLOPPY,
    'Cd': sys_cons.BOOT_SOURCE_TARGET_CD,
    'Usb': sys_cons.BOOT_SOURCE_TARGET_USB,
    'Hdd': sys_cons.BOOT_SOURCE_TARGET_HDD,
    'BiosSetup': sys_cons.BOOT_SOURCE_TARGET_BIOS_SETUP,
    'Utilities': sys_cons.BOOT_SOURCE_TARGET_UTILITIES,
    'Diags': sys_cons.BOOT_SOURCE_TARGET_DIAGS,
    'SDCard': sys_cons.BOOT_SOURCE_TARGET_SD_CARD,
    'UefiTarget': sys_cons.BOOT_SOURCE_TARGET_UEFI_TARGET,
    'UefiShell': sys_cons.BOOT_SOURCE_TARGET_UEFI_SHELL,
    'UefiHttp': sys_cons.BOOT_SOURCE_TARGET_UEFI_HTTP,
}

BOOT_SOURCE_TARGET_MAP_REV = utils.revert_dictionary(BOOT_SOURCE_TARGET_MAP)

BOOT_SOURCE_MODE_MAP = {
    'Legacy': sys_cons.BOOT_SOURCE_MODE_BIOS,
    'UEFI': sys_cons.BOOT_SOURCE_MODE_UEFI,
}

BOOT_SOURCE_MODE_MAP_REV = utils.revert_dictionary(BOOT_SOURCE_MODE_MAP)

BOOT_SOURCE_ENABLED_MAP = {
    'Once': sys_cons.BOOT_SOURCE_ENABLED_ONCE,
    'Continuous': sys_cons.BOOT_SOURCE_ENABLED_CONTINUOUS,
    'Disabled': sys_cons.BOOT_SOURCE_ENABLED_DISABLED,
}

BOOT_SOURCE_ENABLED_MAP_REV = utils.revert_dictionary(BOOT_SOURCE_ENABLED_MAP)

PROCESSOR_ARCH_VALUE_MAP = {
    'x86': sys_cons.PROCESSOR_ARCH_x86,
    'IA-64': sys_cons.PROCESSOR_ARCH_IA_64,
    'ARM': sys_cons.PROCESSOR_ARCH_ARM,
    'MIPS': sys_cons.PROCESSOR_ARCH_MIPS,
    'OEM': sys_cons.PROCESSOR_ARCH_OEM,
}

PROCESSOR_ARCH_VALUE_MAP_REV = (
    utils.revert_dictionary(PROCESSOR_ARCH_VALUE_MAP))

PROCESSOR_TYPE_VALUE_MAP = {
    'Accelerator': sys_cons.PROCESSOR_TYPE_ACCELERATOR,
    'CPU': sys_cons.PROCESSOR_TYPE_CPU,
    'Core': sys_cons.PROCESSOR_TYPE_CORE,
    'DSP': sys_cons.PROCESSOR_TYPE_DSP,
    'FPGA': sys_cons.PROCESSOR_TYPE_FPGA,
    'GPU': sys_cons.PROCESSOR_TYPE_GPU,
    'OEM': sys_cons.PROCESSOR_TYPE_OEM,
    'Thread': sys_cons.PROCESSOR_TYPE_THREAD
}

PROCESSOR_TYPE_VALUE_MAP_REV = (
    utils.revert_dictionary(PROCESSOR_TYPE_VALUE_MAP))

PROCESSOR_INSTRUCTIONSET_VALUE_MAP = {
    'ARM-A32': sys_cons.PROCESSOR_INSTRUCTIONSET_ARM_A32,
    'ARM-A64': sys_cons.PROCESSOR_INSTRUCTIONSET_ARM_A64,
    'IA-64': sys_cons.PROCESSOR_INSTRUCTIONSET_IA_64,
    'MIPS32': sys_cons.PROCESSOR_INSTRUCTIONSET_MIPS32,
    'MIPS64': sys_cons.PROCESSOR_INSTRUCTIONSET_MIPS64,
    'OEM': sys_cons.PROCESSOR_INSTRUCTIONSET_OEM,
    'x86': sys_cons.PROCESSOR_INSTRUCTIONSET_x86,
    'x86-64': sys_cons.PROCESSOR_INSTRUCTIONSET_x86_64
}

PROCESSOR_INSTRUCTIONSET_VALUE_MAP_REV = (
    utils.revert_dictionary(PROCESSOR_INSTRUCTIONSET_VALUE_MAP))

SYSTEM_TYPE_VALUE_MAP = {
    'Physical': sys_cons.SYSTEM_TYPE_PHYSICAL,
    'Virtual': sys_cons.SYSTEM_TYPE_VIRTUAL,
    'OS': sys_cons.SYSTEM_TYPE_OS,
    'PhysicallyPartitioned': sys_cons.SYSTEM_TYPE_PHYSICALLY_PARTITIONED,
    'VirtuallyPartitioned': sys_cons.SYSTEM_TYPE_VIRTUALLY_PARTITIONED,
    'Composed': sys_cons.SYSTEM_TYPE_COMPOSED
}

SYSTEM_TYPE_VALUE_MAP_REV = (
    utils.revert_dictionary(SYSTEM_TYPE_VALUE_MAP))



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

# This is referred from Redfish standard schema.
# http://redfish.dmtf.org/schemas/v1/Volume.v1_0_3.json

import logging

from sushy import exceptions
from sushy.resources import base
from sushy.resources import common
from sushy.resources.system.storage import mappings as store_maps
from sushy import utils

LOG = logging.getLogger(__name__)


class ActionsField(base.CompositeField):
    initialize = common.InitializeActionField('#Volume.Initialize')


class Volume(base.ResourceBase):
    """This class adds the Storage Volume resource"""

    identity = base.Field('Id', required=True)
    """The Volume identity string"""

    name = base.Field('Name')
    """The name of the resource"""

    capacity_bytes = base.Field('CapacityBytes', adapter=utils.int_or_none)
    """The size in bytes of this Volume."""

    volume_type = base.MappedField('VolumeType',
                                   store_maps.VOLUME_TYPE_TYPE_MAP)
    """The type of this volume."""

    encrypted = base.Field('Encrypted', adapter=bool)
    """Is this Volume encrypted."""

    identifiers = common.IdentifiersListField('Identifiers', default=[])
    """The Durable names for the volume."""

    block_size_bytes = base.Field('BlockSizeBytes', adapter=int)
    """The size of the smallest addressable unit of this volume in bytes."""

    operation_apply_time_support = common.OperationApplyTimeSupportField()
    """Indicates if a client is allowed to request for a specific apply
    time of a create, delete, or action operation of a given resource"""

    _actions = ActionsField('Actions', required=True)

    def _get_initialize_action_element(self):
        initialize_action = self._actions.initialize
        if not initialize_action:
            raise exceptions.MissingActionError(action='#Volume.Initialize',
                                                resource=self._path)
        return initialize_action

    def get_allowed_initialize_volume_values(self):
        """Get the allowed values for initializing the volume.

        :returns: A set with the allowed values.
        """
        action = self._get_initialize_action_element()

        if not action.allowed_values:
            LOG.warning('Could not figure out the allowed values for the '
                        'initialize volume action for Volume %s',
                        self.identity)
            return set(store_maps.VOLUME_INIT_TYPE_MAP_REV)

        return set([store_maps.VOLUME_INIT_TYPE_MAP[v] for v in
                    set(store_maps.VOLUME_INIT_TYPE_MAP).
                    intersection(action.allowed_values)])

    def initialize_volume(self, value):
        """Initialize the volume.

        :param value: The InitializeType value.
        :raises: InvalidParameterValueError, if the target value is not
            allowed.
        """
        valid_values = self.get_allowed_initialize_volume_values()
        if value not in valid_values:
            raise exceptions.InvalidParameterValueError(
                parameter='value', value=value, valid_values=valid_values)
        value = store_maps.VOLUME_INIT_TYPE_MAP_REV[value]
        target_uri = self._get_initialize_action_element().target_uri
        self._conn.post(target_uri, data={'InitializeType': value})

    def delete_volume(self, payload=None):
        """Delete the volume.

        :param payload: May contain @Redfish.OperationApplyTime property
        :raises: ConnectionError
        :raises: HTTPError
        """
        self._conn.delete(self._path, data=payload)


class VolumeCollection(base.ResourceCollectionBase):
    """This class represents the Storage Volume collection"""

    @property
    def _resource_type(self):
        return Volume

    @property
    @utils.cache_it
    def volumes_sizes_bytes(self):
        """Sizes of all Volumes in bytes in VolumeCollection resource.

        Returns the list of cached values until it (or its parent resource)
        is refreshed.
        """
        return sorted(vol.capacity_bytes for vol in self.get_members())

    @property
    def max_volume_size_bytes(self):
        """Max size available (in bytes) among all Volume resources.

        Returns the cached value until it (or its parent resource) is
        refreshed.
        """
        return utils.max_safe(self.volumes_sizes_bytes)

    # NOTE(etingof): for backward compatibility
    max_size_bytes = max_volume_size_bytes

    operation_apply_time_support = common.OperationApplyTimeSupportField()
    """Indicates if a client is allowed to request for a specific apply
    time of a create, delete, or action operation of a given resource"""

    def create_volume(self, payload):
        """Create a volume.

        :param payload: The payload representing the new volume to create.
        :raises: ConnectionError
        :raises: HTTPError
        """
        r = self._conn.post(self._path, data=payload)
        location = r.headers.get('Location')
        new_volume = None
        if r.status_code == 201:
            if location:
                self.refresh()
                new_volume = self.get_member(location)
        elif r.status_code == 202:
            # TODO(billdodd): TaskMonitor support to be added in subsequent PR
            pass
        return new_volume



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# Values come from the Redfish ResourceBlock json-schema.
# https://redfish.dmtf.org/schemas/ResourceBlock.v1_3_0.json

# Composition state related constants
COMPOSITION_STATE_COMPOSING = 'Composing'
COMPOSITION_STATE_COMPOSED_AND_AVAILABLE = 'ComposedAndAvailable'
COMPOSITION_STATE_COMPOSED = 'Composed'
COMPOSITION_STATE_UNUSED = 'Unused'
COMPOSITION_STATE_FAILED = 'Failed'
COMPOSITION_STATE_UNAVAILABLE = 'Unavailable'

# Resource Block type related constants
RESOURCE_BLOCK_TYPE_COMPUTE = 'Compute'
RESOURCE_BLOCK_TYPE_PROCESSOR = 'Processor'
RESOURCE_BLOCK_TYPE_MEMORY = 'Memory'
RESOURCE_BLOCK_TYPE_NETWORK = 'Network'
RESOURCE_BLOCK_TYPE_STORAGE = 'Storage'
RESOURCE_BLOCK_TYPE_COMPUTERSYSTEM = 'ComputerSystem'
RESOURCE_BLOCK_TYPE_EXPANSION = 'Expansion'



# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from datetime import datetime
from datetime import timedelta

from dateutil import parser
import mock

from sushy.resources.task_monitor import TaskMonitor
from sushy.tests.unit import base


class TaskMonitorTestCase(base.TestCase):

    def setUp(self):
        super(TaskMonitorTestCase, self).setUp()
        self.conn = mock.Mock()
        self.data = {'fake': 'data'}
        self.http_date = 'Fri, 31 Dec 1999 23:59:59 GMT'
        self.seconds = 120
        self.datetime = parser.parse(self.http_date)
        self.req_headers = {'X-Fake': 'header'}
        self.res_headers1 = {'location': 'https://sample.com/foo/bar',
                             'retry-after': self.http_date}
        self.res_headers2 = {'location': 'https://sample.com/foo/bar',
                             'retry-after': str(self.seconds)}

    def test_task_in_progress(self):
        self.conn.post.return_value.status_code = 202
        self.conn.post.return_value.headers = self.res_headers1.copy()
        self.conn.get.return_value.status_code = 202
        self.conn.get.return_value.headers = self.res_headers1.copy()
        self.conn.get.return_value.json.return_value = {}
        res = self.conn.post(path='fake/path', data=self.data.copy(),
                             headers=self.req_headers.copy())
        tm = TaskMonitor(self.conn, res.headers.get('location'))\
            .set_retry_after(res.headers.get('retry-after'))
        self.assertIsNotNone(tm)
        self.assertTrue(tm.in_progress)

    def test_task_not_in_progress(self):
        self.conn.post.return_value.status_code = 202
        self.conn.post.return_value.headers = self.res_headers1.copy()
        self.conn.get.return_value.status_code = 201
        self.conn.get.return_value.json.return_value = self.data.copy()
        res = self.conn.post(path='fake/path', data=self.data.copy(),
                             headers=self.req_headers.copy())
        tm = TaskMonitor(self.conn, res.headers.get('location'))\
            .set_retry_after(res.headers.get('retry-after'))
        self.assertIsNotNone(tm)
        self.assertFalse(tm.in_progress)

    def test_retry_after_http_date(self):
        self.conn.post.return_value.status_code = 202
        self.conn.post.return_value.headers = self.res_headers1.copy()
        self.conn.get.return_value.status_code = 202
        self.conn.get.return_value.headers = self.res_headers1.copy()
        self.conn.get.return_value.json.return_value = {}
        res = self.conn.post(path='fake/path', data=self.data.copy(),
                             headers=self.req_headers.copy())
        tm = TaskMonitor(self.conn, res.headers.get('location')) \
            .set_retry_after(res.headers.get('retry-after'))
        self.assertIsNotNone(tm)
        self.assertEqual(self.datetime, tm.retry_after)

    def test_retry_after_seconds(self):
        self.conn.post.return_value.status_code = 202
        self.conn.post.return_value.headers = self.res_headers2.copy()
        self.conn.get.return_value.status_code = 202
        self.conn.get.return_value.headers = self.res_headers2.copy()
        self.conn.get.return_value.json.return_value = {}
        start = datetime.now() + timedelta(seconds=self.seconds)
        res = self.conn.post(path='fake/path', data=self.data.copy(),
                             headers=self.req_headers.copy())
        tm = TaskMonitor(self.conn, res.headers.get('location')) \
            .set_retry_after(res.headers.get('retry-after'))
        end = datetime.now() + timedelta(seconds=self.seconds)
        self.assertIsNotNone(tm)
        self.assertTrue(start <= tm.retry_after <= end)

    def test_sleep_for(self):
        self.conn.post.return_value.status_code = 202
        self.conn.post.return_value.headers = self.res_headers2.copy()
        self.conn.get.return_value.status_code = 202
        self.conn.get.return_value.headers = self.res_headers2.copy()
        self.conn.get.return_value.json.return_value = {}
        start = datetime.now()
        res = self.conn.post(path='fake/path', data=self.data.copy(),
                             headers=self.req_headers.copy())
        tm = TaskMonitor(self.conn, res.headers.get('location')) \
            .set_retry_after(res.headers.get('retry-after'))
        self.assertIsNotNone(tm)
        sleep_for = tm.sleep_for
        elapsed = (datetime.now() - start).total_seconds()
        self.assertTrue(self.seconds - elapsed <= sleep_for <= self.seconds)

    def test_response(self):
        self.conn.post.return_value.status_code = 202
        self.conn.post.return_value.headers = self.res_headers1.copy()
        self.conn.get.return_value.status_code = 201
        self.conn.get.return_value.json.return_value = self.data.copy()
        res = self.conn.post(path='fake/path', data=self.data.copy(),
                             headers=self.req_headers.copy())
        tm = TaskMonitor(self.conn, res.headers.get('location')) \
            .set_retry_after(res.headers.get('retry-after'))
        self.assertIsNotNone(tm)
        self.assertFalse(tm.in_progress)
        response = tm.response
        self.assertEqual(201, response.status_code)
        self.assertEqual(self.data.copy(), response.json())



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# Values comes from the Redfish System json-schema 1.0.0:
# http://redfish.dmtf.org/schemas/v1/Manager.v1_0_0.json#/definitions/Manager  # noqa

from sushy.resources import constants as res_cons

# Manager Reset action constants

RESET_MANAGER_GRACEFUL_RESTART = res_cons.RESET_TYPE_GRACEFUL_RESTART
"""Perform a graceful shutdown followed by a restart of the system"""

RESET_MANAGER_FORCE_RESTART = res_cons.RESET_TYPE_FORCE_RESTART
"""Perform an immediate (non-graceful) shutdown, followed by a restart"""

# Manager Type constants

MANAGER_TYPE_MANAGEMENT_CONTROLLER = 'management controller'
"""A controller used primarily to monitor or manage the operation of
   a device or system"""

MANAGER_TYPE_ENCLOSURE_MANAGER = 'enclosure manager'
"""A controller which provides management functions for a chassis
   or group of devices or systems"""

MANAGER_TYPE_BMC = 'bmc'
"""A controller which provides management functions for a single
   computer system"""

MANAGER_TYPE_RACK_MANAGER = 'rack manager'
"""A controller which provides management functions for a whole or part
   of a rack"""

MANAGER_TYPE_AUXILIARY_CONTROLLER = 'auxiliary controller'
"""A controller which provides management functions for a particular
   subsystem or group of devices"""

# Graphical Console constants

GRAPHICAL_CONSOLE_KVMIP = 'graphical console kvmip'
"""Graphical Console connection using a KVM-IP (redirection of Keyboard,
   Video, Mouse over IP) protocol"""

GRAPHICAL_CONSOLE_OEM = 'graphical console oem'
"""Graphical Console connection using an OEM-specific protocol"""

# Serial Console constants

SERIAL_CONSOLE_SSH = 'serial console ssh'
"""Serial Console connection using the SSH protocol"""

SERIAL_CONSOLE_TELNET = 'serial console telnet'
"""Serial Console connection using the Telnet protocol"""

SERIAL_CONSOLE_IPMI = 'serial console ipmi'
"""Serial Console connection using the IPMI Serial-over-LAN (SOL) protocol"""

SERIAL_CONSOLE_OEM = 'serial console oem'
"""Serial Console connection using an OEM-specific protocol"""

# Command Shell constants

COMMAND_SHELL_SSH = 'command shell ssh'
"""Command Shell connection using the SSH protocol"""

COMMAND_SHELL_TELNET = 'command shell telnet'
"""Command Shell connection using the Telnet protocol"""

COMMAND_SHELL_IPMI = 'command shell ipmi'
"""Command Shell connection using the IPMI Serial-over-LAN (SOL) protocol"""

COMMAND_SHELL_OEM = 'command shell oem'
"""Command Shell connection using an OEM-specific protocol"""

# Supported Virtual Media Type constants

VIRTUAL_MEDIA_CD = 'cd'
VIRTUAL_MEDIA_DVD = 'dvd'
VIRTUAL_MEDIA_FLOPPY = 'floppy'
VIRTUAL_MEDIA_USBSTICK = 'usb'

# Connected Via constants

CONNECTED_VIA_APPLET = 'applet'
CONNECTED_VIA_NOT_CONNECTED = 'not_connected'
CONNECTED_VIA_OEM = 'oem'
CONNECTED_VIA_URI = 'uri'



# Copyright 2017 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock

from sushy import auth
from sushy import connector
from sushy import exceptions
from sushy import main
from sushy.tests.unit import base


class BasicAuthTestCase(base.TestCase):

    @mock.patch.object(main, 'Sushy', autospec=True)
    @mock.patch.object(connector, 'Connector', autospec=True)
    def setUp(self, mock_connector, mock_root):
        super(BasicAuthTestCase, self).setUp()
        self.username = 'TestUsername'
        self.password = 'TestP@$$W0RD'
        self.base_auth = auth.BasicAuth(self.username,
                                        self.password)
        self.conn = mock_connector.return_value
        self.root = mock_root.return_value

    def test_init(self):
        self.assertEqual(self.username,
                         self.base_auth._username)
        self.assertEqual(self.password,
                         self.base_auth._password)
        self.assertIsNone(self.base_auth._root_resource)
        self.assertIsNone(self.base_auth._connector)

    def test_set_context(self):
        self.base_auth.set_context(self.root, self.conn)
        self.assertEqual(self.base_auth._root_resource,
                         self.root)
        self.assertEqual(self.base_auth._connector,
                         self.conn)

    def test__do_authenticate_no_context(self):
        self.assertRaises(RuntimeError,
                          self.base_auth.authenticate)

    def test__do_authenticate(self):
        self.base_auth.set_context(self.root, self.conn)
        self.base_auth.authenticate()
        self.conn.set_http_basic_auth.assert_called_once_with(self.username,
                                                              self.password)

    def test_can_refresh_session(self):
        self.assertFalse(self.base_auth.can_refresh_session())

    @mock.patch.object(auth.BasicAuth, 'close', autospec=True)
    def test_context_manager(self, auth_close):
        with auth.BasicAuth(self.username, self.password) as base_auth:
            self.assertEqual(self.username, base_auth._username)
            self.assertEqual(self.password, base_auth._password)
        auth_close.assert_called_once_with(base_auth)


class SessionAuthTestCase(base.TestCase):

    @mock.patch.object(main, 'Sushy', autospec=True)
    @mock.patch.object(connector, 'Connector', autospec=True)
    def setUp(self, mock_connector, mock_root):
        super(SessionAuthTestCase, self).setUp()
        self.username = 'TestUsername'
        self.password = 'TestP@$$W0RD'
        self.sess_key = 'TestingKey'
        self.sess_uri = ('https://testing:8000/redfish/v1/'
                         'SessionService/Sessions/testing')
        self.sess_auth = auth.SessionAuth(self.username,
                                          self.password)
        self.conn = mock_connector.return_value
        self.root = mock_root.return_value

    def test_init(self):
        self.assertEqual(self.username,
                         self.sess_auth._username)
        self.assertEqual(self.password,
                         self.sess_auth._password)
        self.assertIsNone(self.sess_auth._root_resource)
        self.assertIsNone(self.sess_auth._connector)
        self.assertIsNone(self.sess_auth._session_key)
        self.assertIsNone(self.sess_auth._session_resource_id)

    def test_get_session_key(self):
        self.sess_auth._session_key = self.sess_key
        self.assertEqual(self.sess_key,
                         self.sess_auth.get_session_key())

    def test_get_session_resource_id(self):
        self.sess_auth._session_resource_id = self.sess_uri
        self.assertEqual(self.sess_uri,
                         self.sess_auth.get_session_resource_id())

    def test_reset_session_attrs(self):
        self.sess_auth._session_key = self.sess_key
        self.sess_auth._session_resource_id = self.sess_uri
        self.assertEqual(self.sess_uri,
                         self.sess_auth.get_session_resource_id())
        self.assertEqual(self.sess_key,
                         self.sess_auth.get_session_key())
        self.sess_auth.reset_session_attrs()
        self.assertIsNone(self.sess_auth.get_session_resource_id())
        self.assertIsNone(self.sess_auth.get_session_key())

    def test_set_context(self):
        self.sess_auth.set_context(self.root, self.conn)
        self.assertEqual(self.sess_auth._root_resource,
                         self.root)
        self.assertEqual(self.sess_auth._connector,
                         self.conn)

    def test__do_authenticate_no_context(self):
        self.assertRaises(RuntimeError,
                          self.sess_auth.authenticate)

    def test__do_authenticate(self):
        self.assertIsNone(self.sess_auth.get_session_resource_id())
        self.assertIsNone(self.sess_auth.get_session_key())
        mock_sess_serv = mock.Mock()
        mock_sess_serv.create_session.return_value = (self.sess_key,
                                                      self.sess_uri)
        self.root.get_session_service.return_value = mock_sess_serv
        self.sess_auth.set_context(self.root, self.conn)
        self.sess_auth.authenticate()
        self.assertEqual(self.sess_uri,
                         self.sess_auth.get_session_resource_id())
        self.assertEqual(self.sess_key,
                         self.sess_auth.get_session_key())
        self.conn.set_http_session_auth.assert_called_once_with(self.sess_key)

    def test_can_refresh_session(self):
        mock_sess_serv = mock.Mock()
        mock_sess_serv.create_session.return_value = (self.sess_key,
                                                      self.sess_uri)
        self.root.get_session_service.return_value = mock_sess_serv

        self.sess_auth.set_context(self.root, self.conn)
        self.sess_auth.authenticate()

        self.assertTrue(self.sess_auth.can_refresh_session())

    def test_refresh(self):
        self.assertIsNone(self.sess_auth.get_session_resource_id())
        self.assertIsNone(self.sess_auth.get_session_key())
        mock_sess_serv = mock.Mock()
        mock_sess_serv.create_session.return_value = (self.sess_key,
                                                      self.sess_uri)
        self.root.get_session_service.return_value = mock_sess_serv
        self.sess_auth.set_context(self.root, self.conn)
        self.sess_auth.refresh_session()
        self.assertEqual(self.sess_uri,
                         self.sess_auth.get_session_resource_id())
        self.assertEqual(self.sess_key,
                         self.sess_auth.get_session_key())
        self.conn.set_http_session_auth.assert_called_once_with(self.sess_key)

    def test_close_do_nothing(self):
        self.sess_auth._session_key = None
        self.sess_auth.set_context(self.root, self.conn)
        self.sess_auth.close()
        self.conn.delete.assert_not_called()

    def test_close(self):
        self.sess_auth._session_key = self.sess_key
        self.sess_auth._session_resource_id = self.sess_uri
        self.sess_auth.set_context(self.root, self.conn)
        self.sess_auth.close()
        self.conn.delete.assert_called_once_with(self.sess_uri)
        self.assertIsNone(self.sess_auth.get_session_resource_id())
        self.assertIsNone(self.sess_auth.get_session_key())

    @mock.patch.object(auth, 'LOG', autospec=True)
    def test_close_fail(self, mock_LOG):
        self.sess_auth._session_key = self.sess_key
        self.sess_auth._session_resource_id = self.sess_uri
        self.conn.delete.side_effect = (
            exceptions.ServerSideError(
                'DELETE', 'any_url', mock.MagicMock()))

        self.sess_auth.set_context(self.root, self.conn)
        self.sess_auth.close()

        self.assertTrue(mock_LOG.warning.called)
        self.assertIsNone(self.sess_auth.get_session_resource_id())
        self.assertIsNone(self.sess_auth.get_session_key())

    @mock.patch.object(auth.SessionAuth, 'close', autospec=True)
    def test_context_manager(self, auth_close):
        with auth.SessionAuth(self.username, self.password) as session_auth:
            self.assertEqual(self.username, session_auth._username)
            self.assertEqual(self.password, session_auth._password)
        auth_close.assert_called_once_with(session_auth)


class SessionOrBasicAuthTestCase(base.TestCase):

    @mock.patch.object(main, 'Sushy', autospec=True)
    @mock.patch.object(connector, 'Connector', autospec=True)
    def setUp(self, mock_connector, mock_root):
        super(SessionOrBasicAuthTestCase, self).setUp()
        self.username = 'TestUsername'
        self.password = 'TestP@$$W0RD'
        self.sess_key = 'TestingKey'
        self.sess_uri = ('https://testing:8000/redfish/v1/'
                         'SessionService/Sessions/testing')
        self.conn = mock_connector.return_value
        self.root = mock_root.return_value
        self.sess_basic_auth = auth.SessionOrBasicAuth(self.username,
                                                       self.password)

    def test_init(self):
        self.assertEqual(self.username,
                         self.sess_basic_auth._username)
        self.assertEqual(self.password,
                         self.sess_basic_auth._password)
        self.assertIsNone(self.sess_basic_auth._root_resource)
        self.assertIsNone(self.sess_basic_auth._connector)
        self.assertIsNone(self.sess_basic_auth._session_key)
        self.assertIsNone(self.sess_basic_auth._session_resource_id)

    def test_get_session_key(self):
        self.sess_basic_auth._session_key = self.sess_key
        self.assertEqual(self.sess_key,
                         self.sess_basic_auth.get_session_key())

    def test_get_session_resource_id(self):
        self.sess_basic_auth._session_resource_id = self.sess_uri
        self.assertEqual(self.sess_uri,
                         self.sess_basic_auth.get_session_resource_id())

    def test_reset_session_attrs(self):
        self.sess_basic_auth._session_key = self.sess_key
        self.sess_basic_auth._session_resource_id = self.sess_uri
        self.assertEqual(self.sess_uri,
                         self.sess_basic_auth.get_session_resource_id())
        self.assertEqual(self.sess_key,
                         self.sess_basic_auth.get_session_key())
        self.sess_basic_auth.reset_session_attrs()
        self.assertIsNone(self.sess_basic_auth.get_session_resource_id())
        self.assertIsNone(self.sess_basic_auth.get_session_key())

    def test_set_context(self):
        self.sess_basic_auth.set_context(self.root, self.conn)
        self.assertEqual(self.sess_basic_auth._root_resource,
                         self.root)
        self.assertEqual(self.sess_basic_auth._connector,
                         self.conn)

    def test__do_authenticate_no_context(self):
        self.assertRaises(RuntimeError,
                          self.sess_basic_auth.authenticate)

    def test__do_authenticate(self):
        self.assertIsNone(self.sess_basic_auth.get_session_resource_id())
        self.assertIsNone(self.sess_basic_auth.get_session_key())
        mock_sess_serv = mock.Mock()
        mock_sess_serv.create_session.return_value = (self.sess_key,
                                                      self.sess_uri)
        self.root.get_session_service.return_value = mock_sess_serv
        self.sess_basic_auth.set_context(self.root, self.conn)
        self.sess_basic_auth.authenticate()
        self.assertEqual(self.sess_uri,
                         self.sess_basic_auth.get_session_resource_id())
        self.assertEqual(self.sess_key,
                         self.sess_basic_auth.get_session_key())
        self.conn.set_http_session_auth.assert_called_once_with(self.sess_key)

    def test__do_authenticate_for_basic_auth(self):
        self.assertIsNone(self.sess_basic_auth.get_session_resource_id())
        self.assertIsNone(self.sess_basic_auth.get_session_key())
        mock_sess_serv = mock.Mock()
        mock_sess_serv.create_session.side_effect = exceptions.SushyError
        self.root.get_session_service.return_value = mock_sess_serv

        self.sess_basic_auth.set_context(self.root, self.conn)
        self.sess_basic_auth.authenticate()

        self.assertIsNone(self.sess_basic_auth.get_session_resource_id())
        self.assertIsNone(self.sess_basic_auth.get_session_key())
        self.conn.set_http_basic_auth.assert_called_once_with(
            self.username, self.password)

    def test_can_refresh_session(self):
        mock_sess_serv = mock.Mock()
        mock_sess_serv.create_session.return_value = (self.sess_key,
                                                      self.sess_uri)
        self.root.get_session_service.return_value = mock_sess_serv

        self.sess_basic_auth.set_context(self.root, self.conn)
        self.sess_basic_auth.authenticate()

        self.assertTrue(self.sess_basic_auth.can_refresh_session())

    def test_refresh_no_previous_session(self):
        self.assertIsNone(self.sess_basic_auth.get_session_resource_id())
        self.assertIsNone(self.sess_basic_auth.get_session_key())
        self.sess_basic_auth.set_context(self.root, self.conn)
        self.sess_basic_auth.refresh_session()
        self.assertIsNone(self.sess_basic_auth.get_session_resource_id())
        self.assertIsNone(self.sess_basic_auth.get_session_key())
        self.conn.set_http_session_auth.assert_not_called()
        self.conn.set_http_basic_auth.assert_not_called()

    def test_refresh_previous_session_exists(self):
        self.sess_basic_auth._session_key = 'ThisisFirstKey'
        test_url = ('https://testing:8000/redfish/v1/SessionService'
                    '/Sessions/testingfirst')
        self.sess_basic_auth._session_resource_id = test_url
        mock_sess_serv = mock.Mock()
        mock_sess_serv.create_session.return_value = (self.sess_key,
                                                      self.sess_uri)
        self.root.get_session_service.return_value = mock_sess_serv
        self.sess_basic_auth.set_context(self.root, self.conn)
        self.sess_basic_auth.refresh_session()
        self.assertEqual(self.sess_uri,
                         self.sess_basic_auth.get_session_resource_id())
        self.assertEqual(self.sess_key,
                         self.sess_basic_auth.get_session_key())
        self.conn.set_http_session_auth.assert_called_once_with(self.sess_key)

    def test_close_do_nothing(self):
        self.conn.delete.assert_not_called()

    def test_close(self):
        self.sess_basic_auth._session_key = self.sess_key
        self.sess_basic_auth._session_resource_id = self.sess_uri
        self.sess_basic_auth.set_context(self.root, self.conn)
        self.sess_basic_auth.close()
        self.conn.delete.assert_called_once_with(self.sess_uri)
        self.assertIsNone(self.sess_basic_auth.get_session_resource_id())
        self.assertIsNone(self.sess_basic_auth.get_session_key())

    @mock.patch.object(auth.SessionOrBasicAuth, 'close', autospec=True)
    def test_context_manager(self, auth_close):
        with auth.SessionOrBasicAuth(
                self.username, self.password) as session_or_base_auth:
            self.assertEqual(self.username, session_or_base_auth._username)
            self.assertEqual(self.password, session_or_base_auth._password)
        auth_close.assert_called_once_with(session_or_base_auth)



#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

# Sushy Redfish Authentication Modes

import abc
import logging

from sushy import exceptions

LOG = logging.getLogger(__name__)


class AuthBase(object, metaclass=abc.ABCMeta):

    def __init__(self, username=None, password=None):
        """A class representing a base Sushy authentication mechanism

        :param username: User account with admin/server-profile
            access privilege.
        :param password: User account password.
        """
        self._username = username
        self._password = password
        self._root_resource = None
        self._connector = None

    def set_context(self, root_resource, connector):
        """Set the context of the authentication object.

        :param root_resource: Root sushy object
        :param connector: Connector for http connections
        """
        self._root_resource = root_resource
        self._connector = connector
        self._connector.set_auth(self)

    def authenticate(self):
        """Perform authentication.

        :raises: RuntimeError
        """
        if self._root_resource is None or self._connector is None:
            raise RuntimeError('_root_resource / _connector is missing. '
                               'Forgot to call set_context()?')
        self._do_authenticate()

    @abc.abstractmethod
    def _do_authenticate(self):
        """Method to establish a session to a Redfish controller.

        Needs to be implemented by extending auth class,
        because each authentication type will authenticate in its own way.
        """

    @abc.abstractmethod
    def can_refresh_session(self):
        """Method to assert if session based refresh can be done."""

    def close(self):
        """Shutdown Redfish authentication object

        Undoes whatever should be undone to cancel authenticated session.
        """

    def __enter__(self):
        """Allow object to be called with the 'with' statement."""
        return self

    def __exit__(self, exception_type, exception_value, traceback):
        """Allow object to be called with the 'with' statement.

        Allow object to be called with the 'with' statement but
        also ensure we call close method on exit.
        """
        self.close()


class BasicAuth(AuthBase):
    """Basic Authentication class.

       This is a class used to encapsulate a basic authentication session.

       :param username: User account with admin/server-profile
           access privilege.
       :param password: User account password.
    """

    def _do_authenticate(self):
        """Attempts to establish a Basic Authentication Session.

        """
        self._connector.set_http_basic_auth(self._username, self._password)

    def can_refresh_session(self):
        """Method to assert if session based refresh can be done."""
        return False


class SessionAuth(AuthBase):
    """Session Authentication class.

       This is a class used to encapsulate a redfish session.
    """

    def __init__(self, username=None, password=None):
        """A class representing a Session Authentication object.

        :param username: User account with admin/server-profile access
             privilege.
        :param password: User account password.
        """
        self._session_key = None
        """Our Sessions Key"""
        self._session_resource_id = None
        """Our Sessions Unique Resource ID or URL"""

        super(SessionAuth, self).__init__(username,
                                          password)

    def get_session_key(self):
        """Returns the session key.

        :returns: The session key.
        """
        return self._session_key

    def get_session_resource_id(self):
        """Returns the session resource id.

        :returns: The session resource id.
        """
        return self._session_resource_id

    def _do_authenticate(self):
        """Establish a redfish session.

        :raises: MissingXAuthToken
        :raises: ConnectionError
        :raises: AccessError
        :raises: HTTPError
        """
        session_service = self._root_resource.get_session_service()
        session_auth_token, session_uri = (
            session_service.create_session(self._username,
                                           self._password))
        self._session_key = session_auth_token
        self._session_resource_id = session_uri
        self._connector.set_http_session_auth(session_auth_token)

    def can_refresh_session(self):
        """Method to assert if session based refresh can be done."""
        return (self._session_key is not None
                and self._session_resource_id is not None)

    def refresh_session(self):
        """Method to refresh a session to a Redfish controller.

        This method is called to create a new session after
        a session that has already been established
        has timed-out or expired.

        :raises: MissingXAuthToken
        :raises: ConnectionError
        :raises: AccessError
        :raises: HTTPError
        """
        self.reset_session_attrs()
        self._do_authenticate()

    def close(self):
        """Close the Redfish Session.

        Attempts to close an established RedfishSession by
        deleting it from the remote Redfish controller.
        """
        if self._session_resource_id is not None:
            try:
                self._connector.delete(self._session_resource_id)
            except (exceptions.AccessError,
                    exceptions.ServerSideError) as exc:
                LOG.warning('Received exception "%(exception)s" while '
                            'attempting to delete the active session: '
                            '%(session_id)s',
                            {'exception': exc,
                             'session_id': self._session_resource_id})
            self.reset_session_attrs()

    def reset_session_attrs(self):
        """Reset active session related attributes."""
        self._session_key = None
        self._session_resource_id = None


class SessionOrBasicAuth(SessionAuth):

    def __init__(self, username=None, password=None):
        super(SessionOrBasicAuth, self).__init__(username, password)
        self.basic_auth = BasicAuth(username=username, password=password)

    def _do_authenticate(self):
        """Establish a RedfishSession.

        We will attempt to establish a redfish session. If we are unable
        to establish one, fallback to basic authentication.
        """
        try:
            # Attempt session based authentication
            super(SessionOrBasicAuth, self)._do_authenticate()
        except exceptions.SushyError as e:
            LOG.debug('Received exception "%(exception)s" while '
                      'attempting to establish a session. '
                      'Falling back to basic authentication.',
                      {'exception': e})

            # Fall back to basic authentication
            self.reset_session_attrs()
            self.basic_auth.set_context(self._root_resource, self._connector)
            self.basic_auth.authenticate()

    def refresh_session(self):
        """Method to refresh a session to a Redfish controller.

        This method is called to create a new RedfishSession
        if we have previously established a RedfishSession and
        the previous session has timed-out or expired.
        If we did not previously have an established session,
        we simply return our BasicAuthentication requests.Session.
        """
        if self.can_refresh_session():
            super(SessionOrBasicAuth, self).refresh_session()



# -*- coding: utf-8 -*-

# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging

import pbr.version

from sushy.main import Sushy
from sushy.resources.chassis.constants import *  # noqa
from sushy.resources.constants import *  # noqa
from sushy.resources.fabric.constants import *  # noqa
from sushy.resources.manager.constants import *  # noqa
from sushy.resources.system.constants import *  # noqa
from sushy.resources.system.storage.constants import *  # noqa
from sushy.resources.updateservice.constants import *  # noqa

__all__ = ('Sushy',)
__version__ = pbr.version.VersionInfo(
    'sushy').version_string()

# Set the default handler to avoid "No handler found" warnings. See:
# https://docs.python.org/3/howto/logging.html#library-config
logging.getLogger(__name__).addHandler(logging.NullHandler())



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from sushy.resources.chassis import constants as cha_cons

CHASSIS_TYPE_VALUE_MAP = {
    'Rack': cha_cons.CHASSIS_TYPE_RACK,
    'Blade': cha_cons.CHASSIS_TYPE_BLADE,
    'Enclosure': cha_cons.CHASSIS_TYPE_ENCLOSURE,
    'StandAlone': cha_cons.CHASSIS_TYPE_STAND_ALONE,
    'RackMount': cha_cons.CHASSIS_TYPE_RACK_MOUNT,
    'Card': cha_cons.CHASSIS_TYPE_CARD,
    'Cartridge': cha_cons.CHASSIS_TYPE_CARTRIDGE,
    'Row': cha_cons.CHASSIS_TYPE_ROW,
    'Pod': cha_cons.CHASSIS_TYPE_POD,
    'Expansion': cha_cons.CHASSIS_TYPE_EXPANSION,
    'Sidecar': cha_cons.CHASSIS_TYPE_SIDECAR,
    'Zone': cha_cons.CHASSIS_TYPE_ZONE,
    'Sled': cha_cons.CHASSIS_TYPE_SLED,
    'Shelf': cha_cons.CHASSIS_TYPE_SHELF,
    'Drawer': cha_cons.CHASSIS_TYPE_DRAWER,
    'Module': cha_cons.CHASSIS_TYPE_MODULE,
    'Component': cha_cons.CHASSIS_TYPE_COMPONENT,
    'IPBasedDrive': cha_cons.CHASSIS_TYPE_IP_BASED_DRIVE,
    'RackGroup': cha_cons.CHASSIS_TYPE_RACK_GROUP,
    'StorageEnclosure': cha_cons.CHASSIS_TYPE_STORAGE_ENCLOSURE,
    'Other': cha_cons.CHASSIS_TYPE_OTHER,
}

CHASSIS_INTRUSION_SENSOR_MAP = {
    'Normal': cha_cons.CHASSIS_INTRUSION_SENSOR_NORMAL,
    'HardwareIntrusion': cha_cons.CHASSIS_INTRUSION_SENSOR_HARDWARE_INTRUSION,
    'TamperingDetected': cha_cons.CHASSIS_INTRUSION_SENSOR_TAMPERING_DETECTED,
}

CHASSIS_INTRUSION_SENSOR_RE_ARM_MAP = {
    'Manual': cha_cons.CHASSIS_INTRUSION_SENSOR_RE_ARM_MANUAL,
    'Automatic': cha_cons.CHASSIS_INTRUSION_SENSOR_RE_ARM_AUTOMATIC,
}



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# This is referred from Redfish standard schema.
# http://redfish.dmtf.org/schemas/v1/Power.v1_3_0.json

from sushy.resources import base
from sushy.resources.chassis.power import mappings as pow_maps
from sushy.resources import common
from sushy.resources import mappings as res_maps
from sushy import utils


class InputRangeListField(base.ListField):
    """This type describes an input range for a power supply"""

    input_type = base.MappedField('InputType',
                                  pow_maps.POWER_SUPPLY_INPUT_TYPE_MAP)
    """The Input type (AC or DC)"""

    maximum_frequency_hz = base.Field('MaximumFrequencyHz',
                                      adapter=utils.int_or_none)
    """The maximum line input frequency at which this power supply input range
    is effective"""

    maximum_voltage = base.Field('MaximumVoltage', adapter=utils.int_or_none)
    """The maximum line input voltage at which this power supply input range
    is effective"""

    minimum_frequency_hz = base.Field('MinimumFrequencyHz',
                                      adapter=utils.int_or_none)
    """The minimum line input frequency at which this power supply input range
    is effective"""

    minimum_voltage = base.Field('MinimumVoltage', adapter=utils.int_or_none)
    """The minimum line input voltage at which this power supply input range
    is effective"""

    output_wattage = base.Field('OutputWattage', adapter=utils.int_or_none)
    """The maximum capacity of this Power Supply when operating in this input
    range"""


class PowerSupplyListField(base.ListField):
    """The power supplies associated with this Power resource"""

    firmware_version = base.Field('FirmwareVersion')
    """The firmware version for this Power Supply"""

    identity = base.Field('MemberId')
    """Identifier of the Power Supply"""

    indicator_led = base.MappedField('IndicatorLed',
                                     res_maps.INDICATOR_LED_VALUE_MAP)
    """The state of the indicator LED, used to identify the power supply"""

    input_ranges = InputRangeListField('InputRanges', default=[])
    """This is the input ranges that the power supply can use"""

    last_power_output_watts = base.Field('LastPowerOutputWatts',
                                         adapter=utils.int_or_none)
    """The average power output of this Power Supply"""

    line_input_voltage = base.Field('LineInputVoltage',
                                    adapter=utils.int_or_none)
    """The line input voltage at which the Power Supply is operating"""

    line_input_voltage_type = base.MappedField(
        'LineInputVoltageType',
        pow_maps.LINE_INPUT_VOLTAGE_TYPE_MAP)
    """The line voltage type supported as an input to this Power Supply"""

    manufacturer = base.Field('Manufacturer')
    """This is the manufacturer of this power supply"""

    model = base.Field('Model')
    """The model number for this Power Supply"""

    name = base.Field('Name')
    """Name of the Power Supply"""

    part_number = base.Field('PartNumber')
    """The part number for this Power Supply"""

    power_capacity_watts = base.Field('PowerCapacityWatts',
                                      adapter=utils.int_or_none)
    """The maximum capacity of this Power Supply"""

    power_supply_type = base.MappedField('PowerSupplyType',
                                         pow_maps.POWER_SUPPLY_TYPE_MAP)
    """The Power Supply type (AC or DC)"""

    serial_number = base.Field('SerialNumber')
    """The serial number for this Power Supply"""

    spare_part_number = base.Field('SparePartNumber')
    """The spare part number for this Power Supply"""

    status = common.StatusField('Status')
    """Status of the sensor"""


class Power(base.ResourceBase):
    """This class represents a Power resource."""

    identity = base.Field('Id', required=True)
    """Identifier of the resource"""

    name = base.Field('Name', required=True)
    """The name of the resource"""

    power_supplies = PowerSupplyListField('PowerSupplies', default=[])
    """Details of a power supplies associated with this system or device"""



# -*- coding: utf-8 -*-

# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json

import mock

import sushy
from sushy import exceptions
from sushy.resources.chassis import chassis
from sushy.resources.manager import manager
from sushy.resources.system import system
from sushy.tests.unit import base


class ChassisTestCase(base.TestCase):

    def setUp(self):
        super(ChassisTestCase, self).setUp()
        self.conn = mock.Mock()
        with open('sushy/tests/unit/json_samples/chassis.json') as f:
            self.json_doc = json.load(f)

        self.conn.get.return_value.json.return_value = self.json_doc

        self.chassis = chassis.Chassis(self.conn, '/redfish/v1/Chassis/Blade1',
                                       redfish_version='1.8.0')

    def test__parse_attributes(self):
        # | WHEN |
        self.chassis._parse_attributes(self.json_doc)
        # | THEN |
        self.assertEqual('1.8.0', self.chassis.redfish_version)
        self.assertEqual('Blade1', self.chassis.identity)
        self.assertEqual('Blade', self.chassis.name)
        self.assertEqual('Test description', self.chassis.description)
        self.assertEqual('45Z-2381', self.chassis.asset_tag)
        self.assertEqual(sushy.CHASSIS_TYPE_BLADE,
                         self.chassis.chassis_type)
        self.assertEqual('Contoso', self.chassis.manufacturer)
        self.assertEqual('SX1000', self.chassis.model)
        self.assertEqual('529QB9450R6', self.chassis.serial_number)
        self.assertEqual('6914260', self.chassis.sku)
        self.assertEqual('166480-S23', self.chassis.part_number)
        self.assertEqual('FFFFFFFF-FFFF-FFFF-FFFF-FFFFFFFFFFFF',
                         self.chassis.uuid)
        self.assertEqual(sushy.INDICATOR_LED_OFF,
                         self.chassis.indicator_led)
        self.assertEqual(sushy.POWER_STATE_ON,
                         self.chassis.power_state)
        self.assertEqual(sushy.STATE_ENABLED, self.chassis.status.state)
        self.assertEqual(44.45, self.chassis.height_mm)
        self.assertEqual(431.8, self.chassis.width_mm)
        self.assertEqual(711, self.chassis.depth_mm)
        self.assertEqual(15.31, self.chassis.weight_kg)
        self.assertEqual(sushy.HEALTH_OK, self.chassis.status.health)
        self.assertEqual(sushy.CHASSIS_INTRUSION_SENSOR_NORMAL,
                         self.chassis.physical_security.intrusion_sensor)
        self.assertEqual(123,
                         self.chassis.physical_security.intrusion_sensor_number
                         )
        self.assertEqual(sushy.CHASSIS_INTRUSION_SENSOR_RE_ARM_MANUAL,
                         self.chassis.physical_security.intrusion_sensor_re_arm
                         )

    def test_get_allowed_reset_chasis_values(self):
        # | GIVEN |
        expected = {sushy.RESET_TYPE_POWER_CYCLE,
                    sushy.RESET_TYPE_PUSH_POWER_BUTTON,
                    sushy.RESET_TYPE_FORCE_ON, sushy.RESET_TYPE_NMI,
                    sushy.RESET_TYPE_FORCE_RESTART,
                    sushy.RESET_TYPE_GRACEFUL_RESTART, sushy.RESET_TYPE_ON,
                    sushy.RESET_TYPE_FORCE_OFF,
                    sushy.RESET_TYPE_GRACEFUL_SHUTDOWN}
        # | WHEN |
        values = self.chassis.get_allowed_reset_chassis_values()
        # | THEN |
        self.assertEqual(expected, values)
        self.assertIsInstance(values, set)

    def test_get_allowed_reset_chassis_values_for_no_values_set(self):
        # | GIVEN |
        self.chassis._actions.reset.allowed_values = []
        expected = {sushy.RESET_TYPE_POWER_CYCLE,
                    sushy.RESET_TYPE_PUSH_POWER_BUTTON,
                    sushy.RESET_TYPE_FORCE_ON, sushy.RESET_TYPE_NMI,
                    sushy.RESET_TYPE_FORCE_RESTART,
                    sushy.RESET_TYPE_GRACEFUL_RESTART, sushy.RESET_TYPE_ON,
                    sushy.RESET_TYPE_FORCE_OFF,
                    sushy.RESET_TYPE_GRACEFUL_SHUTDOWN}
        # | WHEN |
        values = self.chassis.get_allowed_reset_chassis_values()
        # | THEN |
        self.assertEqual(expected, values)
        self.assertIsInstance(values, set)

    def test_get_allowed_reset_chassis_values_missing_action_reset_attr(self):
        # | GIVEN |
        self.chassis._actions.reset = None
        # | WHEN & THEN |
        self.assertRaisesRegex(
            exceptions.MissingActionError, 'action #Chassis.Reset')

    def test_reset_chassis(self):
        self.chassis.reset_chassis(sushy.RESET_TYPE_GRACEFUL_RESTART)
        self.chassis._conn.post.assert_called_once_with(
            '/redfish/v1/Chassis/Blade1/Actions/Chassis.Reset',
            data={'ResetType': 'GracefulRestart'})

    def test_reset_chassis_with_invalid_value(self):
        self.assertRaises(exceptions.InvalidParameterValueError,
                          self.chassis.reset_chassis, 'invalid-value')

    def test_set_indicator_led(self):
        with mock.patch.object(
                self.chassis, 'invalidate', autospec=True) as invalidate_mock:
            self.chassis.set_indicator_led(sushy.INDICATOR_LED_BLINKING)
            self.chassis._conn.patch.assert_called_once_with(
                '/redfish/v1/Chassis/Blade1',
                data={'IndicatorLED': 'Blinking'})

            invalidate_mock.assert_called_once_with()

    def test_set_indicator_led_invalid_state(self):
        self.assertRaises(exceptions.InvalidParameterValueError,
                          self.chassis.set_indicator_led,
                          'spooky-glowing')

    def test_managers(self):
        # | GIVEN |
        with open('sushy/tests/unit/json_samples/'
                  'manager.json') as f:
            self.conn.get.return_value.json.return_value = json.load(f)

        # | WHEN & THEN |
        actual_managers = self.chassis.managers
        self.assertIsInstance(actual_managers[0], manager.Manager)
        self.assertEqual(
            '/redfish/v1/Managers/Blade1BMC', actual_managers[0].path)

    def test_systems(self):
        # | GIVEN |
        with open('sushy/tests/unit/json_samples/'
                  'system.json') as f:
            self.conn.get.return_value.json.return_value = json.load(f)

        # | WHEN & THEN |
        actual_systems = self.chassis.systems
        self.assertIsInstance(actual_systems[0], system.System)
        self.assertEqual(
            '/redfish/v1/Systems/529QB9450R6', actual_systems[0].path)


class ChassisCollectionTestCase(base.TestCase):

    def setUp(self):
        super(ChassisCollectionTestCase, self).setUp()
        self.conn = mock.Mock()
        with open('sushy/tests/unit/json_samples/'
                  'chassis_collection.json') as f:
            self.conn.get.return_value.json.return_value = json.load(f)
        self.chassis = chassis.ChassisCollection(
            self.conn, '/redfish/v1/Chassis', redfish_version='1.5.0')

    @mock.patch.object(chassis, 'Chassis', autospec=True)
    def test_get_member(self, chassis_mock):
        self.chassis.get_member('/redfish/v1/Chassis/MultiBladeEncl')
        chassis_mock.assert_called_once_with(
            self.chassis._conn, '/redfish/v1/Chassis/MultiBladeEncl',
            self.chassis.redfish_version, None)

    @mock.patch.object(chassis, 'Chassis', autospec=True)
    def test_get_members(self, chassis_mock):
        members = self.chassis.get_members()
        calls = [
            mock.call(self.chassis._conn, '/redfish/v1/Chassis/MultiBladeEncl',
                      self.chassis.redfish_version, None),
            mock.call(self.chassis._conn, '/redfish/v1/Chassis/Blade1',
                      self.chassis.redfish_version, None),
            mock.call(self.chassis._conn, '/redfish/v1/Chassis/Blade2',
                      self.chassis.redfish_version, None),
            mock.call(self.chassis._conn, '/redfish/v1/Chassis/Blade3',
                      self.chassis.redfish_version, None),
            mock.call(self.chassis._conn, '/redfish/v1/Chassis/Blade4',
                      self.chassis.redfish_version, None)
        ]
        chassis_mock.assert_has_calls(calls)
        self.assertIsInstance(members, list)
        self.assertEqual(5, len(members))



# Copyright 2017 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from sushy.resources.fabric import constants as fab_cons
from sushy import utils

ADDRESS_ORIGIN_IPv4_VALUE_MAP = {
    'BOOTP': fab_cons.ADDRESS_ORIGIN_IPv4_BOOTP,
    'DHCP': fab_cons.ADDRESS_ORIGIN_IPv4_DHCP,
    'IPv4LinkLocal': fab_cons.ADDRESS_ORIGIN_IPv4_IPv4LINKLOCAL,
    'Static': fab_cons.ADDRESS_ORIGIN_IPv4_STATIC,
}


ADDRESS_ORIGIN_IPv6_VALUE_MAP = {
    'DHCPv6': fab_cons.ADDRESS_ORIGIN_IPv6_DHCPv6,
    'LinkLocal': fab_cons.ADDRESS_ORIGIN_IPv6_LINKLOCAL,
    'SLAAC': fab_cons.ADDRESS_ORIGIN_IPv6_SLAAC,
    'Static': fab_cons.ADDRESS_ORIGIN_IPv6_STATIC,
}


ADDRESS_STATE_VALUE_MAP = {
    'Deprecated': fab_cons.ADDRESS_STATE_DEPRECATED,
    'Failed': fab_cons.ADDRESS_STATE_FAILED,
    'Preferred': fab_cons.ADDRESS_STATE_PREFERRED,
    'Tentative': fab_cons.ADDRESS_STATE_TENTATIVE,
}


ENTITY_ROLE_VALUE_MAP = {
    'Both': fab_cons.ENTITY_ROLE_BOTH,
    'Initiator': fab_cons.ENTITY_ROLE_INITIATOR,
    'Target': fab_cons.ENTITY_ROLE_TARGET,
}

ENTITY_ROLE_VALUE_MAP_REV = utils.revert_dictionary(ENTITY_ROLE_VALUE_MAP)


ENTITY_TYPE_VALUE_MAP = {
    'Bridge': fab_cons.ENTITY_TYPE_PCI_BRIDGE,
    'DisplayController': fab_cons.ENTITY_TYPE_DISPLAY_CONTROLLER,
    'Drive': fab_cons.ENTITY_TYPE_DRIVE,
    'NetworkController': fab_cons.ENTITY_TYPE_NETWORK_CONTROLLER,
    'Processor': fab_cons.ENTITY_TYPE_PROCESSOR,
    'RootComplex': fab_cons.ENTITY_TYPE_ROOT_COMPLEX,
    'StorageExpander': fab_cons.ENTITY_TYPE_STORAGE_EXPANDER,
    'StorageInitiator': fab_cons.ENTITY_TYPE_STORAGE_INITIATOR,
    'Volume': fab_cons.ENTITY_TYPE_VOLUME,
}

ENTITY_TYPE_VALUE_MAP_REV = utils.revert_dictionary(ENTITY_TYPE_VALUE_MAP)



# Copyright 2017 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json
import mock

from dateutil import parser

import sushy
from sushy import exceptions
from sushy.resources.chassis import chassis
from sushy.resources import constants as res_cons
from sushy.resources.manager import manager
from sushy.resources.oem import fake
from sushy.resources.system import bios
from sushy.resources.system import mappings as sys_map
from sushy.resources.system import processor
from sushy.resources.system import simple_storage
from sushy.resources.system.storage import storage
from sushy.resources.system import system
from sushy.tests.unit import base


class SystemTestCase(base.TestCase):

    def setUp(self):
        super(SystemTestCase, self).setUp()
        self.conn = mock.Mock()
        with open('sushy/tests/unit/json_samples/system.json') as f:
            self.json_doc = json.load(f)

        self.conn.get.return_value.json.return_value = self.json_doc

        self.sys_inst = system.System(
            self.conn, '/redfish/v1/Systems/437XR1138R2',
            redfish_version='1.0.2')

    def test__parse_attributes(self):
        self.sys_inst._parse_attributes(self.json_doc)
        self.assertEqual('1.0.2', self.sys_inst.redfish_version)
        self.assertEqual('Chicago-45Z-2381', self.sys_inst.asset_tag)
        self.assertEqual('P79 v1.33 (02/28/2015)', self.sys_inst.bios_version)
        self.assertEqual('Web Front End node', self.sys_inst.description)
        self.assertEqual('web483', self.sys_inst.hostname)
        self.assertEqual('437XR1138R2', self.sys_inst.identity)
        self.assertEqual(sushy.INDICATOR_LED_OFF,
                         self.sys_inst.indicator_led)
        self.assertEqual('Contoso', self.sys_inst.manufacturer)
        self.assertEqual('WebFrontEnd483', self.sys_inst.name)
        self.assertEqual('224071-J23', self.sys_inst.part_number)
        self.assertEqual('437XR1138R2', self.sys_inst.serial_number)
        self.assertEqual('8675309', self.sys_inst.sku)
        self.assertEqual(sushy.SYSTEM_TYPE_PHYSICAL,
                         self.sys_inst.system_type)
        self.assertEqual('38947555-7742-3448-3784-823347823834',
                         self.sys_inst.uuid)
        self.assertEqual(res_cons.STATE_ENABLED, self.sys_inst.status.state)
        self.assertEqual(res_cons.HEALTH_OK, self.sys_inst.status.health)
        self.assertEqual(res_cons.HEALTH_OK,
                         self.sys_inst.status.health_rollup)
        self.assertEqual(sushy.SYSTEM_POWER_STATE_ON,
                         self.sys_inst.power_state)
        self.assertEqual(96, self.sys_inst.memory_summary.size_gib)
        self.assertEqual("OK", self.sys_inst.memory_summary.health)
        self.assertIsNotNone(self.sys_inst.maintenance_window)
        self.assertEqual(1, self.sys_inst.maintenance_window
                         .maintenance_window_duration_in_seconds)
        self.assertEqual(parser.parse('2016-03-07T14:44:30-05:05'),
                         self.sys_inst.maintenance_window
                         .maintenance_window_start_time)
        for oem_vendor in self.sys_inst.oem_vendors:
            self.assertIn(oem_vendor, ('Contoso', 'Chipwise'))

    def test__parse_attributes_missing_actions(self):
        self.sys_inst.json.pop('Actions')
        self.assertRaisesRegex(
            exceptions.MissingAttributeError, 'attribute Actions',
            self.sys_inst._parse_attributes, self.json_doc)

    def test__parse_attributes_missing_boot(self):
        self.sys_inst.json.pop('Boot')
        self.assertRaisesRegex(
            exceptions.MissingAttributeError, 'attribute Boot',
            self.sys_inst._parse_attributes, self.json_doc)

    def test__parse_attributes_missing_reset_target(self):
        self.sys_inst.json['Actions']['#ComputerSystem.Reset'].pop(
            'target')
        self.assertRaisesRegex(
            exceptions.MissingAttributeError,
            'attribute Actions/#ComputerSystem.Reset/target',
            self.sys_inst._parse_attributes, self.json_doc)

    def test__parse_attributes_null_memory_capacity(self):
        self.sys_inst.json['MemorySummary']['TotalSystemMemoryGiB'] = None
        self.sys_inst._parse_attributes(self.json_doc)
        self.assertIsNone(self.sys_inst.memory_summary.size_gib)

    def test__parse_attributes_bad_maintenance_window_time(self):
        self.sys_inst.json['@Redfish.MaintenanceWindow'][
            'MaintenanceWindowStartTime'] = 'bad date'
        self.assertRaisesRegex(
            exceptions.MalformedAttributeError,
            '@Redfish.MaintenanceWindow/MaintenanceWindowStartTime',
            self.sys_inst._parse_attributes, self.json_doc)

    def test_get__reset_action_element(self):
        value = self.sys_inst._get_reset_action_element()
        self.assertEqual("/redfish/v1/Systems/437XR1138R2/Actions/"
                         "ComputerSystem.Reset",
                         value.target_uri)
        self.assertEqual(["On",
                          "ForceOff",
                          "GracefulShutdown",
                          "GracefulRestart",
                          "ForceRestart",
                          "Nmi",
                          "ForceOn"
                          ],
                         value.allowed_values)

    def test_get__reset_action_element_missing_reset_action(self):
        self.sys_inst._actions.reset = None
        self.assertRaisesRegex(
            exceptions.MissingActionError, 'action #ComputerSystem.Reset',
            self.sys_inst._get_reset_action_element)

    def test_get_allowed_reset_system_values(self):
        values = self.sys_inst.get_allowed_reset_system_values()
        expected = set([sushy.RESET_GRACEFUL_SHUTDOWN,
                        sushy.RESET_GRACEFUL_RESTART,
                        sushy.RESET_FORCE_RESTART,
                        sushy.RESET_FORCE_OFF,
                        sushy.RESET_FORCE_ON,
                        sushy.RESET_ON,
                        sushy.RESET_NMI])
        self.assertEqual(expected, values)
        self.assertIsInstance(values, set)

    @mock.patch.object(system.LOG, 'warning', autospec=True)
    def test_get_allowed_reset_system_values_no_values_specified(
            self, mock_log):
        self.sys_inst._actions.reset.allowed_values = {}
        values = self.sys_inst.get_allowed_reset_system_values()
        # Assert it returns all values if it can't get the specific ones
        expected = set([sushy.RESET_GRACEFUL_SHUTDOWN,
                        sushy.RESET_GRACEFUL_RESTART,
                        sushy.RESET_FORCE_RESTART,
                        sushy.RESET_FORCE_OFF,
                        sushy.RESET_FORCE_ON,
                        sushy.RESET_ON,
                        sushy.RESET_NMI,
                        sushy.RESET_PUSH_POWER_BUTTON])
        self.assertEqual(expected, values)
        self.assertIsInstance(values, set)
        self.assertEqual(1, mock_log.call_count)

    def test_reset_action_operation_apply_time_support(self):
        support = self.sys_inst._actions.reset.operation_apply_time_support
        self.assertIsNotNone(support)
        self.assertEqual(['Immediate', 'AtMaintenanceWindowStart'],
                         support.supported_values)
        self.assertEqual(parser.parse('2017-05-03T23:12:37-05:00'),
                         support.maintenance_window_start_time)
        self.assertEqual(600, support.maintenance_window_duration_in_seconds)
        self.assertEqual('/redfish/v1/Systems/437XR1138R2',
                         support._maintenance_window_resource.resource_uri)

    def test_reset_system(self):
        self.sys_inst.reset_system(sushy.RESET_FORCE_OFF)
        self.sys_inst._conn.post.assert_called_once_with(
            '/redfish/v1/Systems/437XR1138R2/Actions/ComputerSystem.Reset',
            data={'ResetType': 'ForceOff'})

    def test_reset_system_invalid_value(self):
        self.assertRaises(exceptions.InvalidParameterValueError,
                          self.sys_inst.reset_system, 'invalid-value')

    def test_get_allowed_system_boot_source_values(self):
        values = self.sys_inst.get_allowed_system_boot_source_values()
        expected = set([sushy.BOOT_SOURCE_TARGET_NONE,
                        sushy.BOOT_SOURCE_TARGET_PXE,
                        sushy.BOOT_SOURCE_TARGET_CD,
                        sushy.BOOT_SOURCE_TARGET_USB,
                        sushy.BOOT_SOURCE_TARGET_HDD,
                        sushy.BOOT_SOURCE_TARGET_BIOS_SETUP,
                        sushy.BOOT_SOURCE_TARGET_UTILITIES,
                        sushy.BOOT_SOURCE_TARGET_DIAGS,
                        sushy.BOOT_SOURCE_TARGET_SD_CARD,
                        sushy.BOOT_SOURCE_TARGET_UEFI_TARGET])
        self.assertEqual(expected, values)
        self.assertIsInstance(values, set)

    @mock.patch.object(system.LOG, 'warning', autospec=True)
    def test_get_allowed_system_boot_source_values_no_values_specified(
            self, mock_log):
        self.sys_inst.boot.allowed_values = None
        values = self.sys_inst.get_allowed_system_boot_source_values()
        # Assert it returns all values if it can't get the specific ones
        expected = set([sushy.BOOT_SOURCE_TARGET_NONE,
                        sushy.BOOT_SOURCE_TARGET_PXE,
                        sushy.BOOT_SOURCE_TARGET_CD,
                        sushy.BOOT_SOURCE_TARGET_USB,
                        sushy.BOOT_SOURCE_TARGET_HDD,
                        sushy.BOOT_SOURCE_TARGET_BIOS_SETUP,
                        sushy.BOOT_SOURCE_TARGET_UTILITIES,
                        sushy.BOOT_SOURCE_TARGET_DIAGS,
                        sushy.BOOT_SOURCE_TARGET_SD_CARD,
                        sushy.BOOT_SOURCE_TARGET_FLOPPY,
                        sushy.BOOT_SOURCE_TARGET_UEFI_TARGET,
                        sushy.BOOT_SOURCE_TARGET_UEFI_SHELL,
                        sushy.BOOT_SOURCE_TARGET_UEFI_HTTP])
        self.assertEqual(expected, values)
        self.assertIsInstance(values, set)
        self.assertEqual(1, mock_log.call_count)

    def test_set_system_boot_options(self):
        self.sys_inst.set_system_boot_options(
            sushy.BOOT_SOURCE_TARGET_PXE,
            enabled=sushy.BOOT_SOURCE_ENABLED_CONTINUOUS,
            mode=sushy.BOOT_SOURCE_MODE_UEFI)
        self.sys_inst._conn.patch.assert_called_once_with(
            '/redfish/v1/Systems/437XR1138R2',
            data={'Boot': {'BootSourceOverrideEnabled': 'Continuous',
                           'BootSourceOverrideTarget': 'Pxe',
                           'BootSourceOverrideMode': 'UEFI'}})

    def test_set_system_boot_options_no_mode_specified(self):
        self.sys_inst.set_system_boot_options(
            sushy.BOOT_SOURCE_TARGET_HDD,
            enabled=sushy.BOOT_SOURCE_ENABLED_ONCE)
        self.sys_inst._conn.patch.assert_called_once_with(
            '/redfish/v1/Systems/437XR1138R2',
            data={'Boot': {'BootSourceOverrideEnabled': 'Once',
                           'BootSourceOverrideTarget': 'Hdd'}})

    def test_set_system_boot_options_no_target_specified(self):
        self.sys_inst.set_system_boot_options(
            enabled=sushy.BOOT_SOURCE_ENABLED_CONTINUOUS,
            mode=sushy.BOOT_SOURCE_MODE_UEFI)
        self.sys_inst._conn.patch.assert_called_once_with(
            '/redfish/v1/Systems/437XR1138R2',
            data={'Boot': {'BootSourceOverrideEnabled': 'Continuous',
                           'BootSourceOverrideMode': 'UEFI'}})

    def test_set_system_boot_options_no_freq_specified(self):
        self.sys_inst.set_system_boot_options(
            target=sushy.BOOT_SOURCE_TARGET_PXE,
            mode=sushy.BOOT_SOURCE_MODE_UEFI)
        self.sys_inst._conn.patch.assert_called_once_with(
            '/redfish/v1/Systems/437XR1138R2',
            data={'Boot': {'BootSourceOverrideTarget': 'Pxe',
                           'BootSourceOverrideMode': 'UEFI'}})

    def test_set_system_boot_options_nothing_specified(self):
        self.sys_inst.set_system_boot_options()
        self.sys_inst._conn.patch.assert_called_once_with(
            '/redfish/v1/Systems/437XR1138R2', data={})

    def test_set_system_boot_options_invalid_target(self):
        self.assertRaises(exceptions.InvalidParameterValueError,
                          self.sys_inst.set_system_boot_source,
                          'invalid-target')

    def test_set_system_boot_options_invalid_enabled(self):
        with self.assertRaisesRegex(
            exceptions.InvalidParameterValueError,
            '"enabled" value.*{0}'.format(
                list(sys_map.BOOT_SOURCE_ENABLED_MAP_REV))):

            self.sys_inst.set_system_boot_options(
                sushy.BOOT_SOURCE_TARGET_HDD,
                enabled='invalid-enabled')

    def test_set_system_boot_source(self):
        self.sys_inst.set_system_boot_source(
            sushy.BOOT_SOURCE_TARGET_PXE,
            enabled=sushy.BOOT_SOURCE_ENABLED_CONTINUOUS,
            mode=sushy.BOOT_SOURCE_MODE_UEFI)
        self.sys_inst._conn.patch.assert_called_once_with(
            '/redfish/v1/Systems/437XR1138R2',
            data={'Boot': {'BootSourceOverrideEnabled': 'Continuous',
                           'BootSourceOverrideTarget': 'Pxe',
                           'BootSourceOverrideMode': 'UEFI'}})

    def test_set_system_boot_source_no_mode_specified(self):
        self.sys_inst.set_system_boot_source(
            sushy.BOOT_SOURCE_TARGET_HDD,
            enabled=sushy.BOOT_SOURCE_ENABLED_ONCE)
        self.sys_inst._conn.patch.assert_called_once_with(
            '/redfish/v1/Systems/437XR1138R2',
            data={'Boot': {'BootSourceOverrideEnabled': 'Once',
                           'BootSourceOverrideTarget': 'Hdd'}})

    def test_set_system_boot_source_invalid_target(self):
        self.assertRaises(exceptions.InvalidParameterValueError,
                          self.sys_inst.set_system_boot_source,
                          'invalid-target')

    def test_set_system_boot_source_invalid_enabled(self):
        with self.assertRaisesRegex(
            exceptions.InvalidParameterValueError,
            '"enabled" value.*{0}'.format(
                list(sys_map.BOOT_SOURCE_ENABLED_MAP_REV))):

            self.sys_inst.set_system_boot_source(
                sushy.BOOT_SOURCE_TARGET_HDD,
                enabled='invalid-enabled')

    def test_set_indicator_led(self):
        with mock.patch.object(
                self.sys_inst, 'invalidate', autospec=True) as invalidate_mock:
            self.sys_inst.set_indicator_led(sushy.INDICATOR_LED_BLINKING)
            self.sys_inst._conn.patch.assert_called_once_with(
                '/redfish/v1/Systems/437XR1138R2',
                data={'IndicatorLED': 'Blinking'})

            invalidate_mock.assert_called_once_with()

    def test_set_indicator_led_invalid_state(self):
        self.assertRaises(exceptions.InvalidParameterValueError,
                          self.sys_inst.set_indicator_led,
                          'spooky-glowing')

    def test__get_processor_collection_path_missing_processors_attr(self):
        self.sys_inst._json.pop('Processors')
        self.assertRaisesRegex(
            exceptions.MissingAttributeError, 'attribute Processors',
            self.sys_inst._get_processor_collection_path)

    def test_memory_summary_missing_attr(self):
        # | GIVEN |
        self.sys_inst._json['MemorySummary']['Status'].pop('HealthRollup')
        # | WHEN |
        self.sys_inst._parse_attributes(self.json_doc)
        # | THEN |
        self.assertEqual(96, self.sys_inst.memory_summary.size_gib)
        self.assertIsNone(self.sys_inst.memory_summary.health)

        # | GIVEN |
        self.sys_inst._json['MemorySummary'].pop('Status')
        # | WHEN |
        self.sys_inst._parse_attributes(self.json_doc)
        # | THEN |
        self.assertEqual(96, self.sys_inst.memory_summary.size_gib)
        self.assertIsNone(self.sys_inst.memory_summary.health)

        # | GIVEN |
        self.sys_inst._json['MemorySummary'].pop('TotalSystemMemoryGiB')
        # | WHEN |
        self.sys_inst._parse_attributes(self.json_doc)
        # | THEN |
        self.assertIsNone(self.sys_inst.memory_summary.size_gib)
        self.assertIsNone(self.sys_inst.memory_summary.health)

        # | GIVEN |
        self.sys_inst._json.pop('MemorySummary')
        # | WHEN |
        self.sys_inst._parse_attributes(self.json_doc)
        # | THEN |
        self.assertIsNone(self.sys_inst.memory_summary)

    def test_processors(self):
        # | GIVEN |
        self.conn.get.return_value.json.reset_mock()
        with open('sushy/tests/unit/json_samples/'
                  'processor_collection.json') as f:
            self.conn.get.return_value.json.return_value = json.load(f)
        # | WHEN |
        actual_processors = self.sys_inst.processors
        # | THEN |
        self.assertIsInstance(actual_processors,
                              processor.ProcessorCollection)
        self.conn.get.return_value.json.assert_called_once_with()

        # reset mock
        self.conn.get.return_value.json.reset_mock()
        # | WHEN & THEN |
        # tests for same object on invoking subsequently
        self.assertIs(actual_processors,
                      self.sys_inst.processors)
        self.conn.get.return_value.json.assert_not_called()

    def test_processors_on_refresh(self):
        # | GIVEN |
        with open('sushy/tests/unit/json_samples/'
                  'processor_collection.json') as f:
            self.conn.get.return_value.json.return_value = json.load(f)
        # | WHEN & THEN |
        self.assertIsInstance(self.sys_inst.processors,
                              processor.ProcessorCollection)

        # On refreshing the system instance...
        with open('sushy/tests/unit/json_samples/system.json') as f:
            self.conn.get.return_value.json.return_value = json.load(f)

        self.sys_inst.invalidate()
        self.sys_inst.refresh(force=False)

        # | GIVEN |
        with open('sushy/tests/unit/json_samples/'
                  'processor_collection.json') as f:
            self.conn.get.return_value.json.return_value = json.load(f)
        # | WHEN & THEN |
        self.assertIsInstance(self.sys_inst.processors,
                              processor.ProcessorCollection)

    def _setUp_processor_summary(self):
        self.conn.get.return_value.json.reset_mock()
        with open('sushy/tests/unit/json_samples/'
                  'processor_collection.json') as f:
            self.conn.get.return_value.json.return_value = json.load(f)

        # fetch processors for the first time
        self.sys_inst.processors

        successive_return_values = []
        file_names = ['sushy/tests/unit/json_samples/processor.json',
                      'sushy/tests/unit/json_samples/processor2.json']
        for file_name in file_names:
            with open(file_name) as f:
                successive_return_values.append(json.load(f))

        self.conn.get.return_value.json.side_effect = successive_return_values

    def test_processor_summary(self):
        # | GIVEN |
        self._setUp_processor_summary()
        # | WHEN |
        actual_processor_summary = self.sys_inst.processors.summary
        # | THEN |
        self.assertEqual((16, sushy.PROCESSOR_ARCH_x86),
                         actual_processor_summary)
        self.assertEqual(16, actual_processor_summary.count)
        self.assertEqual(sushy.PROCESSOR_ARCH_x86,
                         actual_processor_summary.architecture)

        # reset mock
        self.conn.get.return_value.json.reset_mock()

        # | WHEN & THEN |
        # tests for same object on invoking subsequently
        self.assertIs(actual_processor_summary,
                      self.sys_inst.processors.summary)
        self.conn.get.return_value.json.assert_not_called()

    def test_ethernet_interfaces(self):
        self.conn.get.return_value.json.reset_mock()
        eth_coll_return_value = None
        eth_return_value = None
        with open('sushy/tests/unit/json_samples/'
                  'ethernet_interfaces_collection.json') as f:
            eth_coll_return_value = json.load(f)
        with open('sushy/tests/unit/json_samples/'
                  'ethernet_interfaces.json') as f:
            eth_return_value = json.load(f)

        self.conn.get.return_value.json.side_effect = [eth_coll_return_value,
                                                       eth_return_value]

        actual_macs = self.sys_inst.ethernet_interfaces.summary
        expected_macs = (
            {'12:44:6A:3B:04:11': res_cons.STATE_ENABLED})
        self.assertEqual(expected_macs, actual_macs)

    def test_bios(self):
        self.conn.get.return_value.json.reset_mock()
        bios_return_value = None
        with open('sushy/tests/unit/json_samples/bios.json') as f:
            bios_return_value = json.load(f)
        self.conn.get.return_value.json.side_effect = [bios_return_value]

        self.assertIsInstance(self.sys_inst.bios, bios.Bios)
        self.assertEqual('BIOS Configuration Current Settings',
                         self.sys_inst.bios.name)

    def test_simple_storage_for_missing_attr(self):
        self.sys_inst.json.pop('SimpleStorage')
        with self.assertRaisesRegex(
                exceptions.MissingAttributeError, 'attribute SimpleStorage'):
            self.sys_inst.simple_storage

    def test_simple_storage(self):
        # | GIVEN |
        self.conn.get.return_value.json.reset_mock()
        with open('sushy/tests/unit/json_samples/'
                  'simple_storage_collection.json') as f:
            self.conn.get.return_value.json.return_value = json.load(f)
        # | WHEN |
        actual_simple_storage = self.sys_inst.simple_storage
        # | THEN |
        self.assertIsInstance(actual_simple_storage,
                              simple_storage.SimpleStorageCollection)
        self.conn.get.return_value.json.assert_called_once_with()

        # reset mock
        self.conn.get.return_value.json.reset_mock()
        # | WHEN & THEN |
        # tests for same object on invoking subsequently
        self.assertIs(actual_simple_storage,
                      self.sys_inst.simple_storage)
        self.conn.get.return_value.json.assert_not_called()

    def test_simple_storage_on_refresh(self):
        # | GIVEN |
        with open('sushy/tests/unit/json_samples/'
                  'simple_storage_collection.json') as f:
            self.conn.get.return_value.json.return_value = json.load(f)
        # | WHEN & THEN |
        self.assertIsInstance(self.sys_inst.simple_storage,
                              simple_storage.SimpleStorageCollection)

        # On refreshing the system instance...
        with open('sushy/tests/unit/json_samples/system.json') as f:
            self.conn.get.return_value.json.return_value = json.load(f)

        self.sys_inst.invalidate()
        self.sys_inst.refresh(force=False)

        # | GIVEN |
        with open('sushy/tests/unit/json_samples/'
                  'simple_storage_collection.json') as f:
            self.conn.get.return_value.json.return_value = json.load(f)
        # | WHEN & THEN |
        self.assertIsInstance(self.sys_inst.simple_storage,
                              simple_storage.SimpleStorageCollection)

    def test_storage_for_missing_attr(self):
        self.sys_inst.json.pop('Storage')
        with self.assertRaisesRegex(
                exceptions.MissingAttributeError, 'attribute Storage'):
            self.sys_inst.storage

    def test_storage(self):
        # | GIVEN |
        self.conn.get.return_value.json.reset_mock()
        with open('sushy/tests/unit/json_samples/'
                  'storage_collection.json') as f:
            self.conn.get.return_value.json.return_value = json.load(f)
        # | WHEN |
        actual_storage = self.sys_inst.storage
        # | THEN |
        self.assertIsInstance(actual_storage, storage.StorageCollection)
        self.conn.get.return_value.json.assert_called_once_with()

        # reset mock
        self.conn.get.return_value.json.reset_mock()
        # | WHEN & THEN |
        # tests for same object on invoking subsequently
        self.assertIs(actual_storage, self.sys_inst.storage)
        self.conn.get.return_value.json.assert_not_called()

    def test_storage_on_refresh(self):
        # | GIVEN |
        with open('sushy/tests/unit/json_samples/'
                  'storage_collection.json') as f:
            self.conn.get.return_value.json.return_value = json.load(f)
        # | WHEN & THEN |
        self.assertIsInstance(self.sys_inst.storage,
                              storage.StorageCollection)

        # On refreshing the system instance...
        with open('sushy/tests/unit/json_samples/system.json') as f:
            self.conn.get.return_value.json.return_value = json.load(f)

        self.sys_inst.invalidate()
        self.sys_inst.refresh(force=False)

        # | GIVEN |
        with open('sushy/tests/unit/json_samples/'
                  'storage_collection.json') as f:
            self.conn.get.return_value.json.return_value = json.load(f)
        # | WHEN & THEN |
        self.assertIsInstance(self.sys_inst.storage, storage.StorageCollection)

    def test_managers(self):
        # | GIVEN |
        with open('sushy/tests/unit/json_samples/'
                  'manager.json') as f:
            self.conn.get.return_value.json.return_value = json.load(f)

        # | WHEN & THEN |
        actual_managers = self.sys_inst.managers
        self.assertIsInstance(actual_managers[0], manager.Manager)
        self.assertEqual(
            '/redfish/v1/Managers/BMC', actual_managers[0].path)

    def test_chassis(self):
        # | GIVEN |
        with open('sushy/tests/unit/json_samples/'
                  'chassis.json') as f:
            self.conn.get.return_value.json.return_value = json.load(f)

        # | WHEN & THEN |
        actual_chassis = self.sys_inst.chassis
        self.assertIsInstance(actual_chassis[0], chassis.Chassis)
        self.assertEqual(
            '/redfish/v1/Chassis/1U', actual_chassis[0].path)

    def test_get_oem_extension(self):
        # | WHEN |
        contoso_system_extn_inst = self.sys_inst.get_oem_extension('Contoso')
        # | THEN |
        self.assertIsInstance(contoso_system_extn_inst,
                              fake.FakeOEMSystemExtension)
        self.assertIs(self.sys_inst, contoso_system_extn_inst._parent_resource)
        self.assertEqual('Contoso', contoso_system_extn_inst._vendor_id)


class SystemCollectionTestCase(base.TestCase):

    def setUp(self):
        super(SystemCollectionTestCase, self).setUp()
        self.conn = mock.Mock()
        with open('sushy/tests/unit/json_samples/'
                  'system_collection.json') as f:
            self.json_doc = json.load(f)

        self.conn.get.return_value.json.return_value = self.json_doc

        self.sys_col = system.SystemCollection(
            self.conn, '/redfish/v1/Systems', redfish_version='1.0.2')

    def test__parse_attributes(self):
        self.sys_col._parse_attributes(self.json_doc)
        self.assertEqual('1.0.2', self.sys_col.redfish_version)
        self.assertEqual('Computer System Collection', self.sys_col.name)
        self.assertEqual(('/redfish/v1/Systems/437XR1138R2',),
                         self.sys_col.members_identities)

    @mock.patch.object(system, 'System', autospec=True)
    def test_get_member(self, mock_system):
        self.sys_col.get_member('/redfish/v1/Systems/437XR1138R2')
        mock_system.assert_called_once_with(
            self.sys_col._conn, '/redfish/v1/Systems/437XR1138R2',
            self.sys_col.redfish_version, None)

    @mock.patch.object(system, 'System', autospec=True)
    def test_get_members(self, mock_system):
        members = self.sys_col.get_members()
        mock_system.assert_called_once_with(
            self.sys_col._conn, '/redfish/v1/Systems/437XR1138R2',
            self.sys_col.redfish_version, None)
        self.assertIsInstance(members, list)
        self.assertEqual(1, len(members))



# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


import json
import mock

from sushy.resources.registry import message_registry_file
from sushy.tests.unit import base


class MessageRegistryFileTestCase(base.TestCase):

    def setUp(self):
        super(MessageRegistryFileTestCase, self).setUp()
        self.conn = mock.Mock()
        with open('sushy/tests/unit/json_samples/'
                  'message_registry_file.json') as f:
            self.json_doc = json.load(f)

        self.conn.get.return_value.json.return_value = self.json_doc

        self.reg_file = message_registry_file.MessageRegistryFile(
            self.conn, '/redfish/v1/Registries/Test',
            redfish_version='1.0.2')

    def test__parse_attributes(self):
        self.reg_file._parse_attributes(self.json_doc)
        self.assertEqual('Test', self.reg_file.identity)
        self.assertEqual('Test Message Registry File', self.reg_file.name)
        self.assertEqual('Message Registry file for testing',
                         self.reg_file.description)
        self.assertEqual('en', self.reg_file.languages[0])
        self.assertEqual('Test.1.0', self.reg_file.registry)
        self.assertEqual('default', self.reg_file.location[0].language)
        self.assertEqual('/redfish/v1/Registries/Test/Test.1.0.json',
                         self.reg_file.location[0].uri)
        self.assertEqual('https://example.com/Registries/Test.1.0.json',
                         self.reg_file.location[0].publication_uri)
        self.assertEqual('/redfish/v1/Registries/Archive.zip',
                         self.reg_file.location[0].archive_uri)
        self.assertEqual('Test.1.0.json',
                         self.reg_file.location[0].archive_file)

    @mock.patch('sushy.resources.registry.message_registry.MessageRegistry',
                autospec=True)
    @mock.patch('sushy.resources.base.JsonDataReader', autospec=True)
    def test_get_message_registry_uri(self, mock_reader, mock_msg_reg):
        mock_reader_rv = mock.Mock()
        mock_reader.return_value = mock_reader_rv
        mock_reader_rv.get_json.return_value = {
            "@odata.type": "#MessageRegistry.v1_1_1.MessageRegistry",
        }
        mock_msg_reg_rv = mock.Mock()
        mock_msg_reg.return_value = mock_msg_reg_rv

        registry = self.reg_file.get_message_registry('en', None)
        mock_msg_reg.assert_called_once_with(
            self.conn, path='/redfish/v1/Registries/Test/Test.1.0.json',
            reader=None, redfish_version=self.reg_file.redfish_version)
        self.assertEqual(mock_msg_reg_rv, registry)

    @mock.patch('sushy.resources.registry.message_registry.MessageRegistry',
                autospec=True)
    @mock.patch('sushy.resources.base.JsonArchiveReader', autospec=True)
    def test_get_message_registry_archive(self, mock_reader, mock_msg_reg):
        mock_reader_rv = mock.Mock()
        mock_reader.return_value = mock_reader_rv
        mock_msg_reg_rv = mock.Mock()
        mock_reader_rv.get_json.return_value = {
            "@odata.type": "#MessageRegistry.v1_1_1.MessageRegistry",
        }
        mock_msg_reg.return_value = mock_msg_reg_rv
        self.reg_file.location[0].uri = None

        registry = self.reg_file.get_message_registry('fr', None)
        mock_msg_reg.assert_called_once_with(
            self.conn, path='/redfish/v1/Registries/Archive.zip',
            redfish_version=self.reg_file.redfish_version,
            reader=mock_reader_rv)
        mock_reader.assert_called_once_with('Test.1.0.json')
        self.assertEqual(mock_msg_reg_rv, registry)

    @mock.patch('sushy.resources.registry.message_registry.MessageRegistry',
                autospec=True)
    @mock.patch('sushy.resources.base.JsonPublicFileReader', autospec=True)
    def test_get_message_registry_public(self, mock_reader, mock_msg_reg):
        public_connector = mock.Mock()
        mock_reader_rv = mock.Mock()
        mock_reader.return_value = mock_reader_rv
        mock_msg_reg_rv = mock.Mock()
        mock_reader_rv.get_json.return_value = {
            "@odata.type": "#MessageRegistry.v1_1_1.MessageRegistry",
        }
        mock_msg_reg.return_value = mock_msg_reg_rv
        self.reg_file.location[0].uri = None
        self.reg_file.location[0].archive_uri = None

        registry = self.reg_file.get_message_registry('en', public_connector)
        mock_msg_reg.assert_called_once_with(
            public_connector,
            path='https://example.com/Registries/Test.1.0.json',
            redfish_version=self.reg_file.redfish_version,
            reader=mock_reader_rv)
        self.assertEqual(mock_msg_reg_rv, registry)

    @mock.patch('sushy.resources.registry.message_registry.MessageRegistry',
                autospec=True)
    @mock.patch('sushy.resources.base.JsonDataReader', autospec=True)
    def test_get_message_registry_unknown_type(
            self, mock_reader, mock_msg_reg):
        mock_reader_rv = mock.Mock()
        mock_reader.return_value = mock_reader_rv
        mock_reader_rv.get_json.return_value = {
            "@odata.type": "#FishingRegistry.v1_1_1.FishingRegistry",
        }
        mock_msg_reg_rv = mock.Mock()
        mock_msg_reg.return_value = mock_msg_reg_rv

        registry = self.reg_file.get_message_registry('en', None)
        self.assertFalse(mock_msg_reg.called)
        self.assertIsNone(registry)

    @mock.patch('sushy.resources.registry.message_registry.MessageRegistry',
                autospec=True)
    @mock.patch('sushy.resources.registry.message_registry_file.LOG',
                autospec=True)
    def test_get_message_registry_invalid(self, mock_log, mock_msg_reg):
        mock_msg_reg_rv = mock.Mock()
        mock_msg_reg.return_value = mock_msg_reg_rv
        self.reg_file.location[0].uri = None
        self.reg_file.location[0].archive_uri = None
        self.reg_file.location[0].publication_uri = None

        registry = self.reg_file.get_message_registry('en', None)
        mock_msg_reg.assert_not_called()
        self.assertIsNone(registry)
        mock_log.warning.assert_called_with(
            'No message registry found for %(language)s or default',
            {'language': 'en'})

    @mock.patch('sushy.resources.registry.message_registry.MessageRegistry',
                autospec=True)
    @mock.patch('sushy.resources.registry.message_registry_file.RegistryType',
                autospec=True)
    @mock.patch('sushy.resources.registry.message_registry_file.LOG',
                autospec=True)
    def test_get_message_registry_invalid_uri(
            self, mock_log, mock_msg_reg_type, mock_msg_reg):
        mock_msg_reg_rv = mock.Mock()
        mock_msg_reg.return_value = mock_msg_reg_rv
        self.reg_file.location[0].uri = {'extref': 'http://127.0.0.1/reg'}
        mock_msg_reg.side_effect = TypeError('Wrong URL type')
        mock_msg_reg_type.return_value._odata_type = mock.MagicMock(
            endswith=mock.MagicMock(return_value=True))

        registry = self.reg_file.get_message_registry('en', None)

        self.assertIsNone(registry)

        mock_msg_reg_type.assert_called_once_with(
            mock.ANY,
            path={'extref': 'http://127.0.0.1/reg'}, reader=None,
            redfish_version='1.0.2')

        mock_msg_reg.assert_called_once_with(
            mock.ANY,
            path={'extref': 'http://127.0.0.1/reg'}, reader=None,
            redfish_version='1.0.2')

        expected_calls = [
            mock.call(
                'Cannot load message registry from location %(location)s: '
                '%(error)s',
                {'location': {'extref': 'http://127.0.0.1/reg'},
                 'error': mock.ANY}),
            mock.call(
                'No message registry found for %(language)s or default',
                {'language': 'en'})
        ]

        mock_log.warning.assert_has_calls(expected_calls)

    @mock.patch('sushy.resources.registry.message_registry_file.RegistryType',
                autospec=True)
    def test_get_message_registry_non_default_lang(self, mock_registry_type):
        mock_fishing_registry = mock_registry_type.return_value
        mock_fishing_registry._odata_type = 'FishingRegistry'
        self.reg_file.location[0].language = 'en'
        registry = self.reg_file.get_message_registry('en', None)
        mock_registry_type.assert_called_once_with(
            self.conn, path='/redfish/v1/Registries/Test/Test.1.0.json',
            reader=None, redfish_version=self.reg_file.redfish_version)
        self.assertIsNone(registry)

    @mock.patch('sushy.resources.registry.message_registry_file.RegistryType',
                autospec=True)
    def test_get_message_registry_strangely_cased_lang(
            self, mock_registry_type):
        mock_fishing_registry = mock_registry_type.return_value
        mock_fishing_registry._odata_type = 'FishingRegistry'
        self.reg_file.location[0].language = 'En'
        registry = self.reg_file.get_message_registry('en', None)
        mock_registry_type.assert_called_once_with(
            self.conn, path='/redfish/v1/Registries/Test/Test.1.0.json',
            reader=None, redfish_version=self.reg_file.redfish_version)
        self.assertIsNone(registry)

    @mock.patch('sushy.resources.registry.message_registry.MessageRegistry',
                autospec=True)
    @mock.patch('sushy.resources.registry.message_registry_file.LOG',
                autospec=True)
    def test_get_message_registry_missing_lang(self, mock_log, mock_msg_reg):
        mock_msg_reg_rv = mock.Mock()
        mock_msg_reg.return_value = mock_msg_reg_rv
        self.reg_file.location[0].language = 'cz'

        registry = self.reg_file.get_message_registry('en', None)
        mock_msg_reg.assert_not_called()
        self.assertIsNone(registry)
        mock_log.warning.assert_called_with(
            'No message registry found for %(language)s or default',
            {'language': 'en'})

    @mock.patch('sushy.resources.base.logging.warning',
                autospec=True)
    def test__parse_attributes_missing_registry(self, mock_log):
        self.json_doc.pop('Registry')
        self.reg_file._parse_attributes(self.json_doc)
        self.assertEqual('UNKNOWN.0.0', self.reg_file.registry)
        mock_log.assert_called_with(
            'Applying default "UNKNOWN.0.0" on required, but missing '
            'attribute "[\'Registry\']"')


class MessageRegistryFileCollectionTestCase(base.TestCase):

    def setUp(self):
        super(MessageRegistryFileCollectionTestCase, self).setUp()
        self.conn = mock.Mock()
        with open('sushy/tests/unit/json_samples/'
                  'message_registry_file_collection.json') as f:
            self.json_doc = json.load(f)

        self.conn.get.return_value.json.return_value = self.json_doc

        self.reg_file_col =\
            message_registry_file.MessageRegistryFileCollection(
                self.conn, '/redfish/v1/Registries',
                redfish_version='1.0.2')

    def test__parse_attributes(self):
        self.reg_file_col._parse_attributes(self.json_doc)
        self.assertEqual('1.0.2', self.reg_file_col.redfish_version)
        self.assertEqual('Message Registry Test Collection',
                         self.reg_file_col.name)
        self.assertEqual(('/redfish/v1/Registries/Test',),
                         self.reg_file_col.members_identities)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

# Volume Initialization Types
VOLUME_INIT_TYPE_FAST = 'fast'
"""The volume is prepared for use quickly, typically by erasing just the
beginning and end of the space so that partitioning can be performed."""

VOLUME_INIT_TYPE_SLOW = 'slow'
"""The volume is prepared for use slowly, typically by completely erasing
the volume."""

# VolumeType Types
VOLUME_TYPE_RAW_DEVICE = 'rawdevice'
"""The volume is a raw physical device without any RAID or other
virtualization applied."""

VOLUME_TYPE_NON_REDUNDANT = 'nonredundant'
"""The volume is a non-redundant storage device."""

VOLUME_TYPE_MIRRORED = 'mirrored'
"""The volume is a mirrored device."""

VOLUME_TYPE_STRIPED_WITH_PARITY = 'stripedwithparity'
"""The volume is a device which uses parity to retain redundant information."""

VOLUME_TYPE_SPANNED_MIRRORS = 'spannedmirrors'
"""The volume is a spanned set of mirrored devices."""

VOLUME_TYPE_SPANNED_STRIPES_WITH_PARITY = 'spannedstripeswithparity'
"""The volume is a spanned set of devices which uses parity to retain
redundant information."""



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
import mock

from sushy import exceptions
from sushy.resources import constants as res_cons
from sushy.resources.updateservice import softwareinventory
from sushy.tests.unit import base


class SoftwareInventoryTestCase(base.TestCase):

    def setUp(self):
        super(SoftwareInventoryTestCase, self).setUp()
        conn = mock.Mock()
        with open(
            'sushy/tests/unit/json_samples/softwareinventory.json') as f:
            self.json_doc = json.load(f)

        conn.get.return_value.json.return_value = self.json_doc

        self.soft_inv = softwareinventory.SoftwareInventory(
            conn,
            '/redfish/v1/UpdateService/SoftwareInventory/1',
            redfish_version='1.3.0')

    def test__parse_attributes(self):
        self.soft_inv._parse_attributes(self.json_doc)
        self.assertEqual('BMC', self.soft_inv.identity)
        self.assertEqual(
            '1.30.367a12-rev1',
            self.soft_inv.lowest_supported_version)
        self.assertEqual('Contoso', self.soft_inv.manufacturer)
        self.assertEqual('Contoso BMC Firmware', self.soft_inv.name)
        self.assertEqual('2017-08-22T12:00:00', self.soft_inv.release_date)
        self.assertEqual(
            res_cons.STATE_ENABLED,
            self.soft_inv.status.state)
        self.assertEqual(res_cons.HEALTH_OK, self.soft_inv.status.health)
        self.assertEqual(
            '1624A9DF-5E13-47FC-874A-DF3AFF143089',
            self.soft_inv.software_id)
        self.assertTrue(self.soft_inv.updateable)
        self.assertEqual('1.45.455b66-rev4', self.soft_inv.version)

    def test__parse_attributes_missing_identity(self):
        self.soft_inv.json.pop('Id')
        self.assertRaisesRegex(
            exceptions.MissingAttributeError, 'attribute Id',
            self.soft_inv._parse_attributes, self.json_doc)


class SoftwareInventoryCollectionTestCase(base.TestCase):

    def setUp(self):
        super(SoftwareInventoryCollectionTestCase, self).setUp()
        conn = mock.Mock()
        with open('sushy/tests/unit/json_samples/'
                  'softwareinventory_collection.json') as f:
            self.json_doc = json.load(f)

        conn.get.return_value.json.return_value = self.json_doc

        self.soft_inv_col = softwareinventory.SoftwareInventoryCollection(
            conn, '/redfish/v1/UpdateService/SoftwareInventory',
            redfish_version='1.3.0')

    def test__parse_attributes(self):
        self.soft_inv_col._parse_attributes(self.json_doc)
        self.assertEqual('1.3.0', self.soft_inv_col.redfish_version)
        self.assertEqual(
            'Software Inventory Collection',
            self.soft_inv_col.name)

    @mock.patch.object(
        softwareinventory, 'SoftwareInventory', autospec=True)
    def test_get_member(self, mock_softwareinventory):
        path = '/redfish/v1/UpdateService/SoftwareInventory/1'
        self.soft_inv_col.get_member(path)
        mock_softwareinventory.assert_called_once_with(
            self.soft_inv_col._conn, path,
            self.soft_inv_col.redfish_version, None)



# Copyright 2017 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging
from urllib import parse as urlparse

import requests
import time

from sushy import exceptions
from sushy.resources.task_monitor import TaskMonitor

LOG = logging.getLogger(__name__)


class Connector(object):

    def __init__(
            self, url, username=None, password=None, verify=True,
            response_callback=None):
        self._url = url
        self._verify = verify
        self._session = requests.Session()
        self._session.verify = self._verify
        self._response_callback = response_callback

        # NOTE(etingof): field studies reveal that some BMCs choke at
        # long-running persistent HTTP connections (or TCP connections).
        # By default, we ask HTTP server to shut down HTTP connection we've
        # just used.
        self._session.headers['Connection'] = 'close'

        if username or password:
            LOG.warning('Passing username and password to Connector is '
                        'deprecated. Authentication is passed through '
                        'set_auth now, support for these arguments will '
                        'be removed in the future')
            self.set_http_basic_auth(username, password)

    def set_auth(self, auth):
        """Sets the authentication mechanism for our connector."""
        self._auth = auth

    def set_http_basic_auth(self, username, password):
        """Sets the http basic authentication information."""
        self._session.auth = (username, password)

    def set_http_session_auth(self, session_auth_token):
        """Sets the session authentication information."""
        self._session.auth = None
        self._session.headers.update({'X-Auth-Token': session_auth_token})

    def close(self):
        """Close this connector and the associated HTTP session."""
        self._session.close()

    def _op(self, method, path='', data=None, headers=None, blocking=False,
            timeout=60, **extra_session_req_kwargs):
        """Generic RESTful request handler.

        :param method: The HTTP method to be used, e.g: GET, POST,
            PUT, PATCH, etc...
        :param path: The sub-URI or absolute URL path to the resource.
        :param data: Optional JSON data.
        :param headers: Optional dictionary of headers.
        :param blocking: Whether to block for asynchronous operations.
        :param timeout: Max time in seconds to wait for blocking async call.
        :param extra_session_req_kwargs: Optional keyword argument to pass
         requests library arguments which would pass on to requests session
         object.
        :returns: The response object from the requests library.
        :raises: ConnectionError
        :raises: HTTPError
        """
        url = path if urlparse.urlparse(path).netloc else urlparse.urljoin(
            self._url, path)
        headers = headers or {}
        if not any(k.lower() == 'odata-version' for k in headers):
            headers['OData-Version'] = '4.0'
        # TODO(lucasagomes): We should mask the data to remove sensitive
        # information
        LOG.debug('HTTP request: %(method)s %(url)s; headers: %(headers)s; '
                  'body: %(data)s; blocking: %(blocking)s; timeout: '
                  '%(timeout)s; session arguments: %(session)s;',
                  {'method': method, 'url': url, 'headers': headers,
                   'data': data, 'blocking': blocking, 'timeout': timeout,
                   'session': extra_session_req_kwargs})
        try:
            response = self._session.request(method, url, json=data,
                                             headers=headers,
                                             **extra_session_req_kwargs)
        except requests.ConnectionError as e:
            raise exceptions.ConnectionError(url=url, error=e)

        if self._response_callback:
            self._response_callback(response)

        # If we received an AccessError, and we
        # previously established a redfish session
        # there is a chance that the session has timed-out.
        # Attempt to re-establish a session.
        try:
            exceptions.raise_for_response(method, url, response)
        except exceptions.AccessError:
            if self._auth.can_refresh_session():
                self._auth.refresh_session()
                LOG.debug("Authentication refreshed successfully, "
                          "retrying the call.")
                response = self._session.request(method, url, json=data,
                                                 headers=headers,
                                                 **extra_session_req_kwargs)
            else:
                raise

        if blocking and response.status_code == 202:
            if not response.headers.get('location'):
                m = ('HTTP response for %(method)s request to %(url)s '
                     'returned status 202, but no Location header'
                     % {'method': method, 'url': url})
                raise exceptions.ConnectionError(url=url, error=m)
            timeout_at = time.time() + timeout
            mon = (TaskMonitor(self, response.headers.get('location'))
                   .set_retry_after(response.headers.get('retry-after')))
            while mon.in_progress:
                LOG.debug('Blocking for in-progress %(method)s call to '
                          '%(url)s; sleeping for %(sleep)s seconds',
                          {'method': method, 'url': url,
                           'sleep': mon.sleep_for})
                time.sleep(mon.sleep_for)
                if time.time() >= timeout_at and mon.in_progress:
                    m = ('Timeout waiting for blocking %(method)s '
                         'request to %(url)s (timeout = %(timeout)s)'
                         % {'method': method, 'url': url,
                            'timeout': timeout})
                    raise exceptions.ConnectionError(url=url, error=m)
            response = mon.response

        LOG.debug('HTTP response for %(method)s %(url)s: '
                  'status code: %(code)s',
                  {'method': method, 'url': url,
                   'code': response.status_code})

        return response

    def get(self, path='', data=None, headers=None, blocking=False,
            timeout=60, **extra_session_req_kwargs):
        """HTTP GET method.

        :param path: Optional sub-URI path to the resource.
        :param data: Optional JSON data.
        :param headers: Optional dictionary of headers.
        :param blocking: Whether to block for asynchronous operations.
        :param timeout: Max time in seconds to wait for blocking async call.
        :param extra_session_req_kwargs: Optional keyword argument to pass
         requests library arguments which would pass on to requests session
         object.
        :returns: The response object from the requests library.
        :raises: ConnectionError
        :raises: HTTPError
        """
        return self._op('GET', path, data=data, headers=headers,
                        blocking=blocking, timeout=timeout,
                        **extra_session_req_kwargs)

    def post(self, path='', data=None, headers=None, blocking=False,
             timeout=60, **extra_session_req_kwargs):
        """HTTP POST method.

        :param path: Optional sub-URI path to the resource.
        :param data: Optional JSON data.
        :param headers: Optional dictionary of headers.
        :param blocking: Whether to block for asynchronous operations.
        :param timeout: Max time in seconds to wait for blocking async call.
        :param extra_session_req_kwargs: Optional keyword argument to pass
         requests library arguments which would pass on to requests session
         object.
        :returns: The response object from the requests library.
        :raises: ConnectionError
        :raises: HTTPError
        """
        return self._op('POST', path, data=data, headers=headers,
                        blocking=blocking, timeout=timeout,
                        **extra_session_req_kwargs)

    def patch(self, path='', data=None, headers=None, blocking=False,
              timeout=60, **extra_session_req_kwargs):
        """HTTP PATCH method.

        :param path: Optional sub-URI path to the resource.
        :param data: Optional JSON data.
        :param headers: Optional dictionary of headers.
        :param blocking: Whether to block for asynchronous operations.
        :param timeout: Max time in seconds to wait for blocking async call.
        :param extra_session_req_kwargs: Optional keyword argument to pass
         requests library arguments which would pass on to requests session
         object.
        :returns: The response object from the requests library.
        :raises: ConnectionError
        :raises: HTTPError
        """
        return self._op('PATCH', path, data=data, headers=headers,
                        blocking=blocking, timeout=timeout,
                        **extra_session_req_kwargs)

    def put(self, path='', data=None, headers=None, blocking=False,
            timeout=60, **extra_session_req_kwargs):
        """HTTP PUT method.

        :param path: Optional sub-URI path to the resource.
        :param data: Optional JSON data.
        :param headers: Optional dictionary of headers.
        :param blocking: Whether to block for asynchronous operations.
        :param timeout: Max time in seconds to wait for blocking async call.
        :param extra_session_req_kwargs: Optional keyword argument to pass
         requests library arguments which would pass on to requests session
         object.
        :returns: The response object from the requests library.
        :raises: ConnectionError
        :raises: HTTPError
        """
        return self._op('PUT', path, data=data, headers=headers,
                        blocking=blocking, timeout=timeout,
                        **extra_session_req_kwargs)

    def delete(self, path='', data=None, headers=None, blocking=False,
               timeout=60, **extra_session_req_kwargs):
        """HTTP DELETE method.

        :param path: Optional sub-URI path to the resource.
        :param data: Optional JSON data.
        :param headers: Optional dictionary of headers.
        :param blocking: Whether to block for asynchronous operations.
        :param timeout: Max time in seconds to wait for blocking async call.
        :param extra_session_req_kwargs: Optional keyword argument to pass
         requests library arguments which would pass on to requests session
         object.
        :returns: The response object from the requests library.
        :raises: ConnectionError
        :raises: HTTPError
        """
        return self._op('DELETE', path, data=data, headers=headers,
                        blocking=blocking, timeout=timeout,
                        **extra_session_req_kwargs)

    def __enter__(self):
        return self

    def __exit__(self, *_args):
        self.close()
